#include "JoystickPlayer.h"
#include "config.h"
#include <math.h>

JoystickPlayer::JoystickPlayer(Maze* m, HWND hwnd) {
    maze = m;

    SetStartPosition();

    joystick = new Joystick();

    if (!joystick->Init(hwnd)) {
        valid = false;
    }
}

JoystickPlayer::~JoystickPlayer() {
  printf("deleting JoystickPlayer (%x)\n", this);
  if (joystick) {
        delete joystick;
    }
}

void JoystickPlayer::Update(double elapsed) {
    joystick->Update();

    // Get and scale joystick position
    double joyPos[2];
    joystick->GetPosition(joyPos);

    joyPos[0] *= config::movementScale * elapsed;
    joyPos[1] *= config::movementScale * elapsed;

    double speed = sqrt(joyPos[0]*joyPos[0] + joyPos[1]*joyPos[1]);
    if (speed < config::joystickDeadZone) {
      joyPos[0] = 0.0;
      joyPos[1] = 0.0;
    }

    // Test this move with the maze
    double newPos[2];
    newPos[0] = pos[0] + joyPos[0];
    newPos[1] = pos[2] + joyPos[1];

    //tryToMove(newPos);
    if (!maze->validMove(newPos)) {
        // UGLY!!!!
        double oldPos[2];
        oldPos[0] = pos[0];
        oldPos[1] = pos[2];

        double testPos[2];
        int wall = -1;

        // Test Left and Right
        testPos[0] = newPos[0];
        testPos[1] = oldPos[1];
        if (!maze->validMove(testPos)) {
            if (oldPos[0] > newPos[0]) {
                wall = Joystick::WALL_LEFT;
            }
            else {
                wall = Joystick::WALL_RIGHT;
            }
            oldPos[1] = newPos[1];
        } else {
            // Test Up and Down
            testPos[0] = oldPos[0];
            testPos[1] = newPos[1];
            if (!maze->validMove(testPos)) {
                if (oldPos[1] > newPos[1]) {
                    wall = Joystick::WALL_UP;
                }
                else {
                    wall = Joystick::WALL_DOWN;
                }
                oldPos[0] = newPos[0];
            }
        }

        newPos[0] = oldPos[0];
        newPos[1] = oldPos[1];
        if (wall != -1) joystick->PlayBump(wall);
    }
    tryToMove(newPos);
}
