//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include "Player.h"
#include "speech.h"
#include "jfilesystem.h"
#include "config.h"
#include <math.h>
#include <string>
#include "glwrapper.h"

//////////////////////////////////////////////////////////////////////

using namespace std;
using namespace config;

//////////////////////////////////////////////////////////////////////

Player::Player() {
  printf("Build me a player!\n");
  valid = true;
  mapFinished = false;
  moving = false;

  sweepTimer = 0.0;

  soundWallHit = new SoundEntity(0, 0, 0.0, 0.0, 0.0, 0, "player smack", 0.0, 0.0, 0.0, "player\\drip1.wav");
  soundWalking = new SoundEntity(0, 0, 0.0, 0.0, 0.0, 0, "player footsteps", 0.0, 0.0, 0.0, "sounds\\step2.wav");
  sweepEmpty = new SoundEntity(0, 0, 0.0, 0.0, 0.0, 0, "sweep no hit", 0.0, 0.0, 0.0, "player\\sweep_tap.wav");
  sweepFull = new SoundEntity(0, 0, 0.0, 0.0, 0.0, 0, "sweep no hit", 0.0, 0.0, 0.0, "player\\sweep_smack.wav");
  sweepEmpty->source->pause();
  sweepEmpty->source->setLooping(false);
  sweepFull->source->pause();
  sweepFull->source->setLooping(false);

  sweepState = ssNone;
}

//////////////////////////////////////////////////////////////////////

Player::~Player() {
  printf("deleting Player (%x)\n", this);
  delete sweepFull;  sweepFull = NULL;
  delete sweepEmpty;  sweepEmpty = NULL;
  delete soundWallHit;  soundWallHit = NULL;
  delete soundWalking; soundWalking  = NULL;
}

//////////////////////////////////////////////////////////////////////

void Player::hitWall() {
  if (!soundWallHit->source->isPlaying()) {
    soundWallHit->source->start();
    soundWallHit->source->setVolume(192.0);
  }
}

//////////////////////////////////////////////////////////////////////

void Player::tryToMove(double * newPos) {
    tryToMove(newPos, newPos);
}

//////////////////////////////////////////////////////////////////////

// Hacksville
void Player::tryToMove(double * newDevicePos, double * newPlayerPos) {
  bool wasInEntrance = maze->isEntrance(pos);

  // Test this move with the maze
  if (maze->validMove(newDevicePos)) {
    vel[0] = newPlayerPos[0] - pos[0];
    vel[1] = 0.0;
    vel[2] = newPlayerPos[1] - pos[2];

    // Copy the new position
    pos[0] = newPlayerPos[0];
    pos[2] = newPlayerPos[1];
    
    soundWallHit->source->stop();//setPosition(0);
  } else {
    // PLAY AN UMPH SOUND HERE
    hitWall();
    //vel[0] = 0.0;
    //vel[1] = 0.0;
    //vel[2] = 0.0;
  }

  // check for stuff like the exit
  if (maze->isExit(pos)) {
    // Play a reward sound
    startSpeechEvent("You found the exit");
    TStringList * list = vfsFileSearch("\\rewards\\");
    string reward("rewards\\");
    reward += string(list->Items(rand() % list->Count()));
    startSpeechEvent(reward.c_str());
    delete list;

    soundWalking->source->setVolume(0);

    // Done!
    mapFinished = true;
  }

  // Play a warning if the player just *entered* a start square
  if (maze->isEntrance(pos) && !wasInEntrance) {
    startSpeechEvent("Back at the start");
  }

  // Walking sound
  double speed = sqrt(vel[0]*vel[0] + vel[1]*vel[1] + vel[2]*vel[2]);
  moving = speed > 0.0;
  //printf("\r%f", speed);

  static int xxx = 256;
  if (speed > minVelocityForFeet) {
    if (xxx < 256) xxx += 32;
  } else {
    if (xxx > 0) xxx -= 32;
    //soundWalking->source->setVolume(0);
  }
  soundWalking->source->setVolume(xxx);
  soundWalking->source->setPosition(pos[0], pos[1], pos[2]);
}

//////////////////////////////////////////////////////////////////////

bool Player::IsValid() {
  return valid;
}

//////////////////////////////////////////////////////////////////////

void Player::GetPosition(double* position) {
  position[0] = pos[0];
  position[1] = pos[1];
  position[2] = pos[2];
}

//////////////////////////////////////////////////////////////////////

void Player::SetStartPosition() {
  double start[2];
  maze->getStart(start);
  pos[0] = start[0];
  pos[1] = playerHeight;     // This is the height off the surface in openGL land
  pos[2] = start[1];
}

//////////////////////////////////////////////////////////////////////

void Player::caneSweep(double elapsed) {
  // ugly on so many different levels
  static double test[2];
  double d[3];
  double t;
  int i;

  if (!screenBlanked) {
    glPushMatrix();
    glTranslated(test[0], 0.5, test[1]);
    glutSolidCube(0.5);
    glPopMatrix();
  }

  sweepTimer -= elapsed;
  if (sweepTimer > 0.0) {
    return;
  } else {
    sweepTimer += sweepInterval;
  }

  // Figure out the unit direction vector and quit if we're not moving
  t = 0;
  for (i = 0; i < 3; i++) t += vel[i]*vel[i];
  t = sqrt(t);
  if (t < minVelocityForFeet) return;
  t = 1.0 / t;
  for (i = 0; i < 3; i++) d[i] = vel[i] * t;

  // Figure out what to do
  switch (sweepState) {
    case ssNone:
//      if (keyGetState(caneSweepKey)) sweepState = ssCenter;
//      return;
    case ssCenter:
      sweepState = ssLeft;
      return;
    case ssLeft:
      sweepState = ssMiddle;
      test[0] = pos[0] + d[0]*sweepRange - d[2]*sweepWidth;
      test[1] = pos[2] + d[2]*sweepRange + d[0]*sweepWidth;
      break;
    case ssMiddle:
      sweepState = ssRight;
      return;
    case ssRight:
      sweepState = ssCenter;
      test[0] = pos[0] + d[0]*sweepRange + d[2]*sweepWidth;
      test[1] = pos[2] + d[2]*sweepRange - d[0]*sweepWidth;
      break;
  }

  printf("pos(%f, %f)   test(%f, %f)\n", test[0], test[1], pos[0], pos[2], d[0], d[1]);
  if (maze->validMove(test)) {
    sweepEmpty->source->pause();
    sweepEmpty->source->start();
    sweepEmpty->source->setPosition(test[0], 0.0, test[1]);
    sweepEmpty->source->setLooping(false);
    sweepEmpty->source->setVolume(64.0);
    printf("Yes");
  } else {
    sweepFull->source->pause();
    sweepFull->source->start();
    sweepFull->source->setLooping(false);
    sweepFull->source->setVolume(128.0);
    sweepFull->source->setPosition(test[0], 0.0, test[1]);
    printf("No");
  }

  //this->sweepEmpty
  //soundWallHit->start();
  //soundWallHit->setVolume(64);
}

//////////////////////////////////////////////////////////////////////
