//////////////////////////////////////////////////////////////////////

#include "system.h"
#include "jfilesystem.h"
#include "jstring.h"

//////////////////////////////////////////////////////////////////////
// Globals ///////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

TList mounts;

//////////////////////////////////////////////////////////////////////
// Functions /////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

TVFSStream * vfsFileOpen(const char * FileName, TVFSFileMode Mode) {
  TVFSStream * Result;
  for (int index = 0; index < mounts.Count(); index++) {
    Result = ((TVFSMount *)mounts.Items(index))->fsFileOpen(FileName, Mode);

    if (Result != NULL) {
      if (Result->FValid)
        return Result;
      else {
        delete Result;
      }
    }
  }
  return NULL;
}

//////////////////////////////////////////////////////////////////////

bool vfsFileExists(const char * FileName) {
  bool Result;
  for (int index = 0; index < mounts.Count(); index++) {
    Result = ((TVFSMount *)mounts.Items(index))->fsFileExists(FileName);
    if (Result) return Result;
  }
  return false;
}

//////////////////////////////////////////////////////////////////////

TStringList * vfsFileSearch(const char * Path) {
  TStringList * Result = new TStringList();
  for (int index = 0; index < mounts.Count(); index++)
    ((TVFSMount *)mounts.Items(index))->fsListFiles(Path, Result);
  return Result;
}

//////////////////////////////////////////////////////////////////////
/*
function vfsExtractFileName(const FileName: string): string;
begin
end;

//////////////////////////////////////////////////////////////////////

function vfsExtractFileExt(const FileName: string): string;
begin
end;

//////////////////////////////////////////////////////////////////////

function vfsExtractFilePath(const FileName: string): string;
begin
end;

//////////////////////////////////////////////////////////////////////

function vfsExtractFileDir(const FileName: string): string;
begin
end;

//////////////////////////////////////////////////////////////////////

function vfsExtractFileDrive(const FileName: string): string;
begin
end;

//////////////////////////////////////////////////////////////////////
  */
void vfsTranslatePath(char * Path, char seperator) {
/*  while (true) {
    switch (*Path) {
      case 0: return;
      case '|':
      case '/':
      case '\\':
        *Path = seperator;
        break;
    }
    Path++;
  }*/
  int length = j_stringLength(Path);
  for (int i = 0; i < length; i++)
    switch (Path[i]) {
      case 0: return;
      case '|':
      case '/':
      case '\\':
        Path[i] = seperator;
        break;
    }
}

//////////////////////////////////////////////////////////////////////

void vfsAddMount(TVFSMount * system) {
  if (system->FValid)
    mounts.Add(system);
  else {
    delete system;
  }
}

//////////////////////////////////////////////////////////////////////

bool matchExtension(const char * filename, const char * extension) {
  int fileLength = j_stringLength(filename);
  int extLength = j_stringLength(extension);
  if (fileLength >= extLength) {
    return (j_stringCompare(&(filename[fileLength-extLength]), extension, false, extLength) == 0);
  } else
    return false;
}

//////////////////////////////////////////////////////////////////////
// TVFSStream implementation /////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

int TVFSStream::getPosition(void) {
  ASSERT_VALID_RET("TVFSStream::getPosition", 0);
  return Seek(0, vfsFromCurrent);
}

//////////////////////////////////////////////////////////////////////

void TVFSStream::setPosition(int newPosition) {
  ASSERT_VALID("TVFSStream::setPosition");
  Seek(newPosition, vfsFromBeginning);
}

//////////////////////////////////////////////////////////////////////

int TVFSStream::getSize(void) {
  ASSERT_VALID_RET("TVFSStream::getPosition", 0);
  int oldPosition, Result;

  oldPosition = Seek(0, vfsFromCurrent);
  Result = Seek(0, vfsFromEnd);
  Seek(oldPosition, vfsFromBeginning);

  return Result;
}

//////////////////////////////////////////////////////////////////////

bool TVFSStream::setSize(int newSize) {
  ASSERT_VALID_RET("TVFSStream::setSize", false);
  return false;
}

//////////////////////////////////////////////////////////////////////

#define MaxBufSize 0xF000

int TVFSStream::CopyFrom(TVFSStream * Source, int count) {
  int bufSize, N;
  void * buffer;
  int Result;

  if (count == 0) {
    Source->setPosition(0);
    count = Source->getSize();
  }

  Result = count;
  bufSize = (count > MaxBufSize) ? MaxBufSize : count;

  buffer = new byte [bufSize];
  while (count) {
    N = (count > bufSize) ? bufSize : count;
    Source->Read(buffer, N);
    Write(buffer, N);
    count -= N;
  }
  delete[] buffer;

  return Result;
}

//////////////////////////////////////////////////////////////////////

uint8 TVFSStream::ReadUint8(void) {
  uint8 Result;
  Read(&Result, 1);
  return Result;
}

//////////////////////////////////////////////////////////////////////

uint16 TVFSStream::ReadUint16(void) {
  uint16 Result;
  Read(&Result, 2);
  return localizeLEPair(Result);
}

//////////////////////////////////////////////////////////////////////

uint32 TVFSStream::ReadUint32(void) {
  uint32 Result;
  Read(&Result, 4);
  return localizeLEQuad(Result);
}

//////////////////////////////////////////////////////////////////////

int8 TVFSStream::ReadInt8(void) {
  int8 Result;
  Read(&Result, 1);
  return Result;
}

//////////////////////////////////////////////////////////////////////

int16 TVFSStream::ReadInt16(void) {
  int16 Result;
  Read(&Result, 2);
  return localizeLEPair(Result);
}

//////////////////////////////////////////////////////////////////////

int32 TVFSStream::ReadInt32(void) {
  int32 Result;
  Read(&Result, 4);
  return localizeLEQuad(Result);
}

//////////////////////////////////////////////////////////////////////
