//////////////////////////////////////////////////////////////////////

#ifndef JFILESYSTEM_H
#define JFILESYSTEM_H

//////////////////////////////////////////////////////////////////////

#include "system.h"
#include "jLists.h"
#include "jstring.h"

//////////////////////////////////////////////////////////////////////
// Constants /////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#define vfsOpenRead        0x0000
#define vfsOpenWrite       0x0001
#define vfsOpenReadWrite   0x0002
#define vfsShareExclusive  0x0010
#define vfsShareDenyWrite  0x0020
#define vfsShareDenyRead   0x0030
#define vfsShareDenyNone   0x0040
#define vfsCreate          0xFFFF

#define faReadOnly  0x00000001
#define faHidden    0x00000002
#define faSysFile   0x00000004
#define faVolumeID  0x00000008
#define faDirectory 0x00000010
#define faArchive   0x00000020
#define faAnyFile   0x0000003F

//////////////////////////////////////////////////////////////////////
// Types /////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

typedef enum {vfsFromBeginning, vfsFromCurrent, vfsFromEnd} TVFSOrigin;
typedef int TVFSFileMode;

//////////////////////////////////////////////////////////////////////
// TVFSStream ////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

class TVFSStream {
 public:
  virtual int Read(void * buffer, int count) = 0;
  virtual int Write(const void * buffer, int count) = 0;
  virtual int Seek(int offset, TVFSOrigin origin) = 0;
  int CopyFrom(TVFSStream * source, int count);

  int getSize();
  virtual bool setSize(int newSize);

  int getPosition();
  void setPosition(int newPosition);

  uint8  ReadUint8();
  uint16 ReadUint16();
  uint32 ReadUint32();
  int8   ReadInt8();
  int16  ReadInt16();
  int32  ReadInt32();

  bool FValid;
};

//////////////////////////////////////////////////////////////////////
// TVFSMount /////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

class TVFSMount {
 public:
  virtual TVFSStream * fsFileOpen(const char * FileName, TVFSFileMode Mode) = 0;
  virtual void fsListFiles(const char * Path, TStringList * list) = 0;
  virtual bool fsFileExists(const char * FileName) = 0;

  bool FValid;
};

//////////////////////////////////////////////////////////////////////
// Function prototypes ///////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

TVFSStream * vfsFileOpen(const char * FileName, TVFSFileMode Mode);
bool vfsFileExists(const char * FileName);
TStringList * vfsFileSearch(const char * Path);

/*
function vfsCreateFile(const FileName: string): boolean;
function vfsDeleteFile(const FileName: string): boolean;
function vfsRenameFile(const OldName, NewName: string): boolean;
function vfsCreateDirectory(const Dir: string): boolean;
function vfsDeleteDirectory(const Dir: string): boolean;}

// my name system: //mount/path/file
function vfsExtractFileName(const FileName: string): string;
function vfsExtractFileExt(const FileName: string): string;
function vfsExtractFilePath(const FileName: string): string;
function vfsExtractFileDir(const FileName: string): string;
function vfsExtractFileDrive(const FileName: string): string;
*/

bool matchExtension(const char * filename, const char * extension);
void vfsTranslatePath(char * Path, char seperator);
void vfsAddMount(TVFSMount * system);

//////////////////////////////////////////////////////////////////////

#endif