//////////////////////////////////////////////////////////////////////

#include <math.h>
#include <stdlib.h>

#include "jparser.h"

//////////////////////////////////////////////////////////////////////

bool TLineParser::initialize(TVFSStream * stream) {
  release();

  // Make sure we have a kosher stream
  if (!stream) return false;
  bufferSize = stream->getSize();
  if (bufferSize <= 0) return false;

  // Copy the stream into our buffer (size+1 to allow a final NUL)
  buffer = new char[bufferSize+1];
  stream->Read(buffer, bufferSize);
  bufferIndex = 0;

  // Yay, return success
  FValid = true;
  return FValid;
}

//////////////////////////////////////////////////////////////////////

void TLineParser::release(void) {
  if (!FValid) return;

  delete [] buffer;
  bufferSize = 0;
  bufferIndex = 0;

  FValid = false;
}

//////////////////////////////////////////////////////////////////////

bool TLineParser::isEmpty(void) {
  return bufferIndex >= bufferSize;
}

//////////////////////////////////////////////////////////////////////

char TLineParser::lookahead(void) {
  if (bufferIndex < bufferSize)
    return buffer[bufferIndex];
  else
    return '\0';
}

//////////////////////////////////////////////////////////////////////

inline void TLineParser::chewWhitespace(void) {
  while (bufferIndex < bufferSize) {
    char c = buffer[bufferIndex];
    if (c > 32) return;
    bufferIndex++;
  }
}

//////////////////////////////////////////////////////////////////////

const char * TLineParser::readString(void) {
  int index, start;

  // Quit now if the buffer is empty
  if (bufferIndex >= bufferSize) return NULL;

  // Kill preceeding whitespace
  chewWhitespace();

  // Find the next whitespace (or end of buffer)
  index = start = bufferIndex;
  while (index < bufferSize) {
    char c = buffer[index];
    if (c <= 32) break;
    index++;
  }

  // Cap off the string (ok even when index == bufferSize, see init)
  buffer[index] = 0;
  bufferIndex = index+1;

  // Return the string
  return &(buffer[start]);
}

//////////////////////////////////////////////////////////////////////

double TLineParser::readFloat(void) {
  const char * s = readString();
  if (s != NULL)
    return atof(s);
  else
    return 0.0;
}

//////////////////////////////////////////////////////////////////////

int TLineParser::readInteger(void) {
  // Kill preceeding whitespace
  chewWhitespace();

  int result = 0;
  while (bufferIndex < bufferSize) {
    unsigned int c = buffer[bufferIndex] - '0';
    if (c < 10) result = result*10 + c; else break;
    bufferIndex++;
  }

  // cap it off
  if (bufferIndex < bufferSize) {
    buffer[bufferIndex] = 0;
    bufferIndex++;
  }

  return result;
}

//////////////////////////////////////////////////////////////////////

const char * TLineParser::readLine(void) {
  int index, start;

  // Quit now if the buffer is empty
  if (bufferIndex >= bufferSize) return NULL;

  // Find the end of the current line (or end of buffer)
  index = start = bufferIndex;
  while (index < bufferSize) {
    char c = buffer[index];
    if ((c == 13) || (c == 10) || (c == 0)) {
      // Kill any additional EOL characters (so we're mac/linux/pc independant)
      while (index < bufferSize) {
        c = buffer[index+1];  // again, good to have that one extra byte!
        if ((c != 13) && (c != 10) && (c != 0)) break;
        index++;
      }
      break;
    }
    index++;
  }

  // Cap off the string (ok even when index == bufferSize, see init)
  buffer[index] = 0;
  bufferIndex = index+1;

  // Return the string
  return &(buffer[start]);
}

//////////////////////////////////////////////////////////////////////