//////////////////////////////////////////////////////////////////////

#ifndef JPARSER_H
#define JPARSER_H

//////////////////////////////////////////////////////////////////////

#include "system.h"
#include "jfilesystem.h"

//////////////////////////////////////////////////////////////////////

// Magical super happy fun time line parser (TLineParser)
//
// This guy creates a buffer from a stream and does non-backtrackable
// parsing into tokens (strings, integers, or floats), which are
// seperated by whitespace (ASCII 0..32). Sure it's stupid, but it is
// flexible enough for most ASCII data file parsing (like .ASE), and
// it is FAST! (well, a great deal faster than naive delphi dynstring
// parsing, that flexibility incurs a terible performance penalty)
//
// You can also grab a line of text at a time (hence the name).
// Just because I'm a really nice guy, it has a lookahead() func.
//
// Internally, it sticks /0's at the end of each token in the buffer
// and just throws back a const char * directly into the buffer, thus
// token references are ONLY VALID FOR THE LIFE OF THE PARSER!  Once
// the buffer is empty, it will return NULL, 0.0, or 0 as the token.
//
// You can check for end-of-buffer using isEmpty()

class TLineParser : public TJObject {
protected:
  char * buffer;
  int bufferSize, bufferIndex;
  inline void chewWhitespace(void);
public:
  bool initialize(TVFSStream * stream);
  virtual void release(void);

  void reset(void) { bufferIndex = 0; }
  bool isEmpty(void);
  char lookahead(void);

  const char * readString(void);
  double readFloat(void);
  int readInteger(void);
  const char * readLine(void);
};

//////////////////////////////////////////////////////////////////////

#endif

//////////////////////////////////////////////////////////////////////