//////////////////////////////////////////////////////////////////////

#include "jstring.h"

//////////////////////////////////////////////////////////////////////


// Copies up to maxLength bytes from source to dest, stopping when
// a NUL is encountered in the source string.
void j_stringCopy(const char * source, char * dest, int maxLength) {
  for (; (*source) && maxLength; maxLength--) *dest++ = *source++;
  if (maxLength) *dest = 0;
}

//////////////////////////////////////////////////////////////////////

void j_stringFill(char * target, char filler, int numToWrite) {
  for (; numToWrite; numToWrite--) *target++ = filler;
}

//////////////////////////////////////////////////////////////////////

void j_stringCombine(const char * s1, const char * s2, char * dest, 
                     int maxLength)
{
  for (; (*s1) && maxLength; maxLength--) *dest++ = *s1++;
  for (; (*s2) && maxLength; maxLength--) *dest++ = *s2++;
  if (maxLength) *dest = 0;
}

//////////////////////////////////////////////////////////////////////

void j_stringInsert(const char * source, const char * istring, 
                    char * dest, int ipoint, int maxLength)
{
  // Copy the first half of source
  for (ipoint--; (*source) && maxLength && ipoint; maxLength--, ipoint--)
    *dest++ = *source++;

  // Copy the insert string
  for (; (*istring) && maxLength; maxLength--)
    *dest++ = *istring++;

  // Copy the rest of source
  for (; (*source) && maxLength; maxLength--)
    *dest++ = *source++;

  if (maxLength) *dest = 0;
}

//////////////////////////////////////////////////////////////////////

// < 0 string1 less than string2 
// 0 string1 identical to string2 
// > 0 string1 greater than string2
int j_stringCompare(const char * string1, const char * string2, 
                    bool caseSensitive, int maxLength)
{
  // passing a length of zero indicates we don't know/care
  if (maxLength < 1) maxLength = -1;

  char s, d;

  do {
    s = *string1++;
    d = *string2++;

    if (!caseSensitive) {
      if ((s >= 'A') && (s <= 'Z')) s += 0x20;
      if ((d >= 'A') && (d <= 'Z')) d += 0x20;
    }

    maxLength--;
  } while ((s == d) && s && d && (maxLength != 0));
  
  return d-s;
}

//////////////////////////////////////////////////////////////////////

int j_stringLength(const char * string1) {
  int i = 0;
  for (; *string1; i++) string1++;
  return i;
}

//////////////////////////////////////////////////////////////////////
