//////////////////////////////////////////////////////////////////////

#include <windows.h>

#include "glwrapper.h"
#include "maze.h"
#include "Player.h"
#include "PhantomPlayer.h"
#include "JoystickPlayer.h"
#include "KeyPlayer.h"
#include "MazeGraphics.h"
#include "MazeGraphicsHaptics.h"

#include "system.h"
#include "inputSystem.h"
#include "fonts.h"
#include "viewpoint.h"
#include <math.h>
#include "vfsWin32.h"
#include "menus.h"
#include "speech.h"
#include "sound.h"
#include "viewpoint.h"
#include "menus.h"
#include "jmath.h"

#include "config.h"

#include <string>

using namespace std;

//////////////////////////////////////////////////////////////////////

// Global variables 
TStringList * mazeList;

//MapSound * tester[4];

Player * player;
Maze * maze;
MazeGraphics * mazeG;
float musicVolume;
float voiceVolume;
const char appName[] = "aMaze Simulator";

MenuSystem * menu;
TViewpoint * mainWindow;
uint32 lastTime;

//////////////////////////////////////////////////////////////////////

typedef enum { gmMainMenu, gmPlaying, gmTransition } TGameMode;
TGameMode gameMode = gmMainMenu;
TGameMode nextMode; // used when in gmTransition
float transitionTimer; // used when in gmTransition

//////////////////////////////////////////////////////////////////////

void loadMaze(char * filename) {
  if (filename == NULL) return;
  // Load Maze
  maze = new Maze(string(".\\base\\mazes\\") + string(filename));

  //if (player) delete player;
  //if (mazeG) delete mazeG;

  // Try Phantom first
#ifdef PHANTOM_ENABLED
  player = new PhantomPlayer(maze);
  if (player->IsValid()) {
    // Need graphic and haptic maze
    mazeG = new MazeGraphicsHaptics(maze, player);
    return;
  } else {
    delete player;
    player = NULL;
  }
#endif    
    // No Phantom, try Joystick

  // Joystick player needs window handle
  HWND hwnd;
  hwnd = FindWindow("GLUT", appName);

  player = new JoystickPlayer(maze, hwnd);

  if (!player->IsValid()) {
    delete player;
    player = NULL;

    // No Joystick, use Keyboard
    player = new KeyPlayer(maze);
  }

  // Just need a graphics maze
  mazeG = new MazeGraphics(maze, player);
}

//////////////////////////////////////////////////////////////////////
// Menu related code /////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

void loadMaze(jMenuItem * sender, MenuSystem * system, int tag) {
  loadMaze(mazeList->Items(tag));
    //"C:\\projects\\comp290\\base\\mazes\\U_curve_tricky.txt");//mazeList->Items(tag));
  gameMode = gmPlaying;
  //startSpeechEvent("Welcome to the maze");
  startSpeechEvent(maze->description.c_str());
}

void editMaze(jMenuItem * sender, MenuSystem * system, int tag) { j_printf("Sorry, not supported yet\n"); }
void quitGame(jMenuItem * sender, MenuSystem * system, int tag) { keyPressEvent(config::exitKey); }

void buildMazeList(jMenuItem * root) {
  char buffer[2048];
  char buffer2[2048];

  if (mazeList == NULL) mazeList = vfsFileSearch("\\mazes\\");

  for (int i = 0; i < mazeList->Count(); i++) {
    // fixme, tres ugly
    strcpy(buffer, mazeList->Items(i));
    char * ptr;
    do {
      ptr = strstr(buffer, "_");
      if (ptr != NULL) *ptr = ' ';
    } while (ptr != NULL);

    ptr = strstr(buffer, ".");
    if (ptr != NULL) {
      if (stricmp(ptr, ".maze") != 0) continue; // only txt format mazes for now
      *ptr = '\0';
    }

    // Long name (spoken when the menu is entered) should be: loading maze named <filename>
    strcpy(buffer2, "loading maze named ");
    strcat(buffer2, buffer);

    // Add the file to the menu
    root->add(buffer, loadMaze, i, buffer2);
  }
}

//////////////////////////////////////////////////////////////////////

void buildMenu(void) {
  jMenuItem * mChangeSettings;
  jMenuItem * mQuit;
  jMenuItem * mMain;
  jMenuItem * mLoadMaze;

  mChangeSettings = new jMenuItem("Settings", "You can change settings here");
  mChangeSettings->add(new mLeftRightWidget("Change Voice Volume", &voiceVolume), 0);
  mChangeSettings->add(new mLeftRightWidget("Change Music Volume", &musicVolume), 0);

  mLoadMaze = new jMenuItem("Load Maze", "Use up and down to list mazes, and right to select");
  buildMazeList(mLoadMaze);

  mQuit = new jMenuItem("Quit ET Maze", "Are you sure you want to quit");
  mQuit->add("Yes", quitGame, 0);
  mQuit->add("No", MenuSystem::goBack, 0);

  mMain = new jMenuItem("aMaze", "Use up and down to list choices, and right to select");
  //mMain->add("Load Maze", loadMaze, 0);
  mMain->add(mLoadMaze, 0);
  //mMain->add("Edit Maze", editMaze, 0);
//  mMain->add(mChangeSettings);
  //mMain->add(new jMenuItem("Credits", mListCredits));
  mMain->add(mQuit, 0);

  menu->initialize(mMain);
}

//////////////////////////////////////////////////////////////////////

void renderThingy(float elapsed) {
  masterTick += elapsed;

  // Render the menu
  glDisable(GL_LIGHTING);
  glDisable(GL_CULL_FACE);
  glEnable(GL_TEXTURE_2D);
  mainWindow->renderMode2D();
  menu->render(mainWindow, elapsed);
  glDisable(GL_TEXTURE_2D);
  glEnable(GL_LIGHTING);
//  glEnable(GL_CULL_FACE);
}

//////////////////////////////////////////////////////////////////////

void loadResources(void) {
  mainWindow = new TViewpoint();
  mainWindow->theta = 0.0f;
  mainWindow->phi = 0.0f;
  mainWindow->pos[0] = -5.0f;
  mainWindow->pos[1] = -5.0f;
  mainWindow->pos[2] = -5.0f;

  mainWindow->setDimensions(screenWidth, screenHeight);
  mainWindow->setPosition(0, 0);

  // Load the font
  qfont = new TFont("qfont.tga");

  // Build the menu
  menu = new MenuSystem();
  buildMenu();
}

//////////////////////////////////////////////////////////////////////

// GLUT callback for redrawing the view 
void glutDisplay() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // Compute the elapsed time (in seconds)
  float elapsed = sysTimeElapsed(&lastTime)*0.001;
  if (elapsed > config::maxTimestep) elapsed = config::maxTimestep;

  if (keyTriggered(config::exitKey)) exit(0);
  if (keyTriggered(config::skipKey)) stopOneSpeechEvent();
  if (keyTriggered(config::screenBlankKey)) config::screenBlanked = !config::screenBlanked;

  pollSpeechEvents();

  // Draw
  switch (gameMode) {
    case gmPlaying:
      if (maze == NULL) {
        printf("Error: gmPlaying with no maze");
        break;
      }
      mainWindow->renderMode3D(0.1f, maze->getWidth() * 2.0f);
      if (keyTriggered(config::zoneSayKey)) {
        // say the zone
        double pos[3];
        player->GetPosition(pos);
        Zone * zone = maze->getZone(pos);
        if (zone != NULL) {
          startSpeechEvent(zone->name.c_str());
        }
      }

      // Update player position
      player->Update(elapsed);
      mazeG->Update(elapsed);
      player->caneSweep(elapsed);

      // Update sound
      {
        float pos[3];
        float up[3];
        float forward[3];
        double gpos[3];
        player->GetPosition(gpos);
        pos[0] = gpos[0];
        pos[1] = gpos[1];
        pos[2] = gpos[2];
        /*if (player->IsMoving()) {
          forward[0] = player->vel[0];
          forward[1] = player->vel[1];
          forward[2] = player->vel[2];
        } else {*/
          forward[0] = 0.0;
          forward[1] = 0.0;
          forward[2] = 1.0;
        //}
        up[0] = 0.0;
        up[1] = 1.0;
        up[2] = 0.0;

  /*if (keyTriggered('Z')) {
    tester[0]->source->start();
    tester[0]->setPosition(j_random(-10.0, 20.0), j_random(-10.0, 20.0), j_random(-10.0, 20.0));  // forward
  }*/

        // fixme: forward should be set according to the freaking forward
        soundMoveListener(pos, NULL, forward, up);
      }

      // Check for maze completion
      if (player->completedMaze()) {
        gameMode = gmTransition;
        nextMode = gmMainMenu;
        transitionTimer = 2.0;
        menu->reactivateMenu();
        if (player) delete player;
        if (mazeG) delete mazeG;
        if (maze) delete maze;
        player = NULL;
        maze = NULL;
        mazeG = NULL;
      }
      break;
    case gmMainMenu:
      renderThingy(elapsed);
      break;
    case gmTransition:
      if (transitionTimer > elapsed) {
        transitionTimer -= elapsed;
      } else {
        transitionTimer = 0.0;
        gameMode = nextMode;
      }
      break;
  }

  glutSwapBuffers();
}

//////////////////////////////////////////////////////////////////////

// GLUT callback for reshaping the window. This is the main place where the 
// viewing and workspace transforms get initialized.
void glutReshape(int width, int height) {   
  screenWidth = width;
  screenHeight = height;
  if (mainWindow) mainWindow->setDimensions(width, height);
}

//////////////////////////////////////////////////////////////////////

// GLUT callback for idle state. Use this as an opportunity to request a redraw.
// Also check for HLAPI errors that have occurred since the last idle check.
void glutIdle() {    
  glutPostRedisplay();
}

//////////////////////////////////////////////////////////////////////
// Input handlers ////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

int glutToKeyCode(int key) {
  switch (key) {
    case GLUT_KEY_LEFT:  return LEFT_ARROW;
    case GLUT_KEY_RIGHT: return RIGHT_ARROW;
    case GLUT_KEY_UP:    return UP_ARROW;
    case GLUT_KEY_DOWN:  return DOWN_ARROW;
    default:             return 0;
  }
}

//////////////////////////////////////////////////////////////////////

void onSpecialKeyPressed(int key, int x, int y) {
  key = glutToKeyCode(key);
  if (key > 0) keyPressEvent(key);

  if ((gameMode == gmPlaying) && (player != NULL))
    player->KeyPress(key);
}

//////////////////////////////////////////////////////////////////////

void onSpecialKeyReleased(int key, int x, int y) {
  key = glutToKeyCode(key);
  if (key > 0) keyReleaseEvent(key);
}

//////////////////////////////////////////////////////////////////////

void onKeyPressed(unsigned char key, int x, int y) {
  if (isalpha(key)) key = toupper(key);
  keyPressEvent(key);
}
 
//////////////////////////////////////////////////////////////////////

void onKeyReleased(unsigned char key, int x, int y) {
  if (isalpha(key)) key = toupper(key);
  keyReleaseEvent(key);
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

 #include <crtdbg.h>
int main(int argc, char *argv[]) {
  _CrtSetDbgFlag(_CRTDBG_CHECK_ALWAYS_DF);
  mazeList = NULL;

  uint32 start = sysGetTime();
  vfsAddMount(new TVFSMountWin32(".\\base"));

  soundInit();
  speechInit();

  gameMode = gmMainMenu;

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);

  // List files in the root of the vfs namespace
/*  j_printf("Files in the root of the vfs namespace:\n");
  TStringList * list = vfsFileSearch("\\");
  for (int i = 0; i < list->Count(); i++) {
    j_printf("  %s\n", list->Items(i));
  }*/

  qfont = NULL;
  mainWindow = NULL;
  menu = NULL;
  maze = NULL;
  mazeG = NULL;
  player = NULL;

  glutInitWindowSize(512, 512);
  glutCreateWindow(appName);

  // Set glut callback functions.
  glutDisplayFunc(glutDisplay);
  glutReshapeFunc(glutReshape);
  glutIdleFunc(glutIdle);
  glutKeyboardFunc(onKeyPressed);
  glutKeyboardUpFunc(onKeyReleased);
  glutSpecialFunc(onSpecialKeyPressed);
  glutSpecialUpFunc(onSpecialKeyReleased);

  loadResources();

//  loadMaze("C:\\projects\\comp290\\base\\mazes\\U_curve_tricky.txt");

  // Start up the main loop
  lastTime = sysGetTime();
  srand(lastTime);
  glutMainLoop();

  // Fixme: glutMainLoop() never returns, none of this gets done!!!!!!!!
  printf("Stuff was fine\n");
  speechCleanup();
  soundCleanup();

  delete menu;
  delete qfont;
  delete mainWindow;
  delete mazeList;

  return 0;
}

//////////////////////////////////////////////////////////////////////
