//////////////////////////////////////////////////////////////////////

#include "maze.h"

//////////////////////////////////////////////////////////////////////

using namespace std;

//////////////////////////////////////////////////////////////////////

const int MAXNAMELENGTH = 1000;

int Maze::getHeight() {
  return height;
}

//////////////////////////////////////////////////////////////////////

int Maze::getWidth() {
  return width;
}

//////////////////////////////////////////////////////////////////////

Cell * Maze::getCell(int x, int y) {
  if (y < 0) y = 0;
  if (y >= height) y = height - 1;
  if (x < 0) x = 0;
  if (x >= width) x = width - 1;
  return maze[y * width + x];
}

//////////////////////////////////////////////////////////////////////

Zone * Maze::getZone(int x, int y) {
  int uid = getCell(x, y)->getZoneUID();
  for (vector<Zone *>::iterator i = zones.begin(); i < zones.end(); i++) {
    if ((*i)->uid == uid) return *i;
  }
  return NULL;
}

//////////////////////////////////////////////////////////////////////

void Maze::getStart(double* start) {
    for (int i = 0; i < width; i++) {
        for (int j = 0; j < height; j++) {
            if (getCell(i, j)->getStatus() == Cell::START) {
                start[0] = i + 0.5;
                start[1] = j + 0.5;
                return;
            }
        }
    }
    start[0] = 1.5;
    start[1] = 1.5;
}

//////////////////////////////////////////////////////////////////////

void Maze::init(int width, int height) {
  if ((width > 0) && (height > 0)) {
    maze = new PCell[width * height];
    for (int i = 0; i < width * height; i++) maze[i] = new Cell();
  }
}

//////////////////////////////////////////////////////////////////////

Maze::Maze(string input) {
//  clear();
	ifstream inFile;
	//int width, heigth;
	try {
		inFile.open(input.c_str());
		if (!inFile)
			throw input;
		//float flizoat;
		//inFile >> flizoat;
		//inFile >>


		char fileVersionString[9];
		inFile.getline(fileVersionString, 9);
		if (strcmp("AMAZEV.2", fileVersionString)) {  //check version string
			int version = 2;
			throw version;
		}

		inFile >> width;		//get maze width 
		inFile >> height;		//get maze height
                init(width, height);

                char dummy;
		dummy = inFile.get();

		for (int c = 0; c < height; c++) {
			char currentChar;
			for (int i = 0; i < width; i ++) {
				currentChar = inFile.get();
				if (currentChar == '\n')
					throw currentChar;
				//Cell someCell (currentChar);
                                maze[c*width+i]->setStatus(currentChar);
			}
			currentChar = inFile.get();
			if (currentChar != '\n')
				throw currentChar;
		}
		float cellSize;
		inFile >> cellSize;
		int zoneNum;
		inFile >> zoneNum;
		//zones.resize(zoneNum);
		int uid;
		unsigned int color;
		string name;
		char currentChar[MAXNAMELENGTH];
		for (int i = 0; i < zoneNum; i++) {
			inFile >> uid;
			inFile >> color;
			inFile.ignore(1, ' ');
			inFile.getline(currentChar, MAXNAMELENGTH, '\n');
                        zones.push_back(new Zone (uid, color, currentChar));
		}

		for (int c = 0; c < height; c++) {
			int currentZoneNum;
			for (int i = 0; i < width; i ++) {
				inFile >> currentZoneNum;
				maze[c*width+i]->setZoneUID(currentZoneNum);
			}
			//currentChar = inFile.get();
			inFile.ignore(1000, '\n');
		}

		int soundNum;
		inFile >> soundNum;
//		sounds.resize(soundNum);
		
		//stuff to be set in the loop
		int someUid, triggered;
		float volume, x, y, z, cutoff, radius;
		unsigned int someColor;
		char someName[MAXNAMELENGTH];
		char fileName[MAXNAMELENGTH];

		for (int c = 0; c < soundNum; c++) { //each sound requires 2 lines
			inFile >> someUid >> someColor >> x >> z >> y >> triggered;
			inFile.ignore(1, ' ');
			inFile.getline(someName, MAXNAMELENGTH);
			inFile >> volume >> cutoff >> radius;
			inFile.ignore(1, ' ');
			inFile.getline(fileName, MAXNAMELENGTH);
                        sounds.push_back(new SoundEntity(someUid, someColor, x, y, z, triggered, someName, volume, cutoff, radius, fileName));
		}

		// ignore the uid sequence
		inFile.ignore(1000, '\n');

                //get description
		char tempChar = inFile.get();
                description = "";
		while (!inFile.eof()) {
			description += tempChar;
			tempChar = inFile.get();
		}

        } catch (string input) {
		cout << "File: " << input << " not found.  " << endl;
	} catch (char unexpected) {
		if (unexpected == '\n')
			cout << "Found unexpected newline character (you should ensure that lines 2 through " 
				 << height+1 << " of file: " << input << "have exactly " << width << " characters.  " << endl;
		else if (isAllowed(unexpected)) {//need some way to tell what characters are allowed
			cout << "Found unexpected character.  There must be more than "
				 << width << " characters in of the lines between 2 and " << height+1 << ".  " << endl;
		}
		else if (unexpected == ' ')
			cout << "Found unexpected space character.  " << endl;
		else
			cout << "Found unexpected character: " << unexpected << ".  " << endl;
	} catch (int version) {
		cout << "This is Version " << version << " of aMaze, please ensure that the maze file you are trying to load is also version" << version << ".  " << endl;
	}
}

//////////////////////////////////////////////////////////////////////

bool Maze::isAllowed (char c) {
  return (c == Cell::EMPTY || c == Cell::EXIT || c == Cell::START);
}

//////////////////////////////////////////////////////////////////////

bool Maze::validMove(double* pos) {
    int x = (int)pos[0];
    int y = (int)pos[1];

    if (isAllowed(getCell(x, y)->getStatus())) {
        return true;
    } else {
        return false;
    }
}

//////////////////////////////////////////////////////////////////////

bool Maze::isExit(double * pos) {
  return (getCell((int)pos[0], (int)pos[2])->getStatus() == Cell::EXIT);
}

//////////////////////////////////////////////////////////////////////

bool Maze::isEntrance(double * pos) {
  return (getCell((int)pos[0], (int)pos[2])->getStatus() == Cell::START);
}

//////////////////////////////////////////////////////////////////////

float Maze::getCellSize () {
  return cellSize;
}

//////////////////////////////////////////////////////////////////////

void Maze::setCellSize (float size) {
  cellSize = size;
}

//////////////////////////////////////////////////////////////////////

Maze::~Maze() {
  printf("deleting Maze (%x)\n", this);

//for (int i = 0; i < width * height; i++) delete maze[i];
  delete [] maze;

  for (vector<Zone *>::iterator i = zones.begin(); i < zones.end(); i++) {
    delete *i;
  }
  zones.clear();

  for (vector<SoundEntity *>::iterator i = sounds.begin(); i < sounds.end(); i++) {
    delete *i;
  }
  sounds.clear();
}

//////////////////////////////////////////////////////////////////////