//////////////////////////////////////////////////////////////////////

#include "menus.h"
#include "inputSystem.h"
#include "config.h"
#include "fonts.h"
#include "glwrapper.h"
#include <algorithm>

//////////////////////////////////////////////////////////////////////

using namespace std;

//////////////////////////////////////////////////////////////////////

bool MenuSystem::initialize(jMenuItem * root) {
  release();

  // Take over management of the root
  rootMenu = root;
//  rootMenu->addReference();

  myIdleTimeout = config::idleTimeout;

  // Set up the current pointers
  reactivateMenu();

  // Yay!
  FValid = true;
  return FValid;
}

//////////////////////////////////////////////////////////////////////

void MenuSystem::release(void) {
  if (FValid) delete rootMenu;//rootMenu->removeReference();
  FValid = false;
}

//////////////////////////////////////////////////////////////////////

void MenuSystem::render(TViewpoint * context, float elapsed) {
  ASSERT_VALID("MenuSystem::render()");
  bool didSomething = false;

  // Render the menu
  if (!config::screenBlanked) {
    if (activeMenu) {
      int menuHeight = (int)activeMenu->children.size() + 2;
      int h = qfont->getTextHeight("@") * 3/2;
      int y = (screenHeight-menuHeight*h)>>1;

      qfont->lock();
      glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
      qfont->drawText(CENTER_JUSTIFY, y, 0, activeMenu->myLongName->c_str());  y += h;
      qfont->drawText(CENTER_JUSTIFY, y, 0, "...");                   y += h;
      for (jMenuList::iterator i = activeMenu->children.begin(); i != activeMenu->children.end(); i++) {
        if (*i == activeItem)
          glColor4f(1.0f, 0.4f, 0.4f, 1.0f);
        else
          glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
        qfont->drawText(CENTER_JUSTIFY, y, 0, (*i)->myShortName->c_str());     y += h;

      }
      qfont->unlock();
    }
  }

  // Menu input is protected by a timeout between key events
  if (myTimeout > elapsed) {
    myTimeout -= elapsed;
    return;
  } else
    myTimeout = 0.0f;

  // Check for input
  if (keyGetState(DOWN_ARROW) || (myIdleTimeout == 0.0f)) {
    didSomething = true;
    stopSpeechEvents();

    // Cycle up through the current menu
    if (activeMenu) {
      jMenuList::iterator i = find(activeMenu->children.begin(), activeMenu->children.end(), activeItem);
      if (i != activeMenu->children.end()) i++;
      if (i == activeMenu->children.end()) {
        i = activeMenu->children.begin();
        startSpeechEvent("End of List");
      }
      changeActiveItem(*i);
    }
    myIdleTimeout = config::idleTimeout;
  }

  if (keyGetState(UP_ARROW)) {
    didSomething = true;
    stopSpeechEvents();

    // Cycle down through the current menu
    if (activeMenu) {
      jMenuList::iterator i = find(activeMenu->children.begin(), activeMenu->children.end(), activeItem);
      if (i == activeMenu->children.end()) {
        // wasn't found, use the first element always
        i = activeMenu->children.begin();
      } else {
        if (i == activeMenu->children.begin()) {
          i = activeMenu->children.end();
          startSpeechEvent("End of List");
        }
        i--;
      }
      changeActiveItem(*i);
    }
  }

  if (keyGetState(LEFT_ARROW)) {
    didSomething = true;
    stopSpeechEvents();
    goBack(NULL, this, 0);
  }

  if (keyGetState(RIGHT_ARROW)) {
    didSomething = true;
    stopSpeechEvents();

    // Go down into the menu / activate a menu item
    if (activeItem) {
      activeItem->activate(this);
      if (!activeItem->children.empty()) {
        menuHistory.push_back(activeItem);
        changeActiveMenu(activeItem);
        changeActiveItem(*(activeItem->children.begin()));
      }
    }
  }

  // Re-activate the key-press timeout
  if (didSomething) {
    myTimeout = config::keypressTimeout;
    myIdleTimeout = config::idleTimeout;
  } else {
     if (myIdleTimeout > elapsed) {
      myIdleTimeout -= elapsed;  // fixme: net effective timeout of up to idle + regular
    } else
      myIdleTimeout = 0.0f;
  }
}

//////////////////////////////////////////////////////////////////////

void MenuSystem::goBack(jMenuItem * sender, MenuSystem * system, int tag) {
#define this system 
  // Go back up the menu
  size_t size = this->menuHistory.size();
  if (size == 1) {
    this->menuHistory.pop_back();
    this->changeActiveMenu(this->rootMenu);
    this->changeActiveItem(*(this->activeMenu->children.begin()));
  } else if (size > 1) {
    this->menuHistory.pop_back();
    this->changeActiveMenu(this->menuHistory.back());
    this->changeActiveItem(*(this->activeMenu->children.begin()));
  }
#undef this
}

//////////////////////////////////////////////////////////////////////

void MenuSystem::changeActiveMenu(jMenuItem * newItem) {
  if (activeMenu != newItem) {
    activeMenu = newItem;
    startSpeechEvent(activeMenu->myLongName->c_str());
  }
}

//////////////////////////////////////////////////////////////////////

void MenuSystem::reactivateMenu(void) {
  activeMenu = NULL;
  activeItem = NULL;

  // Set up the current pointers
  changeActiveMenu(rootMenu);
  changeActiveItem(*(rootMenu->children.begin()));
}

//////////////////////////////////////////////////////////////////////

void MenuSystem::changeActiveItem(jMenuItem * newItem) {
  if (activeItem != newItem) {
    activeItem = newItem;
    startSpeechEvent(activeItem->myShortName->c_str());
  }
}

//////////////////////////////////////////////////////////////////////
