//////////////////////////////////////////////////////////////////////
// sound.cpp - sound player //////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Includes //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#include "sound.h"
#include "jfilesystem.h"

//////////////////////////////////////////////////////////////////////
// Sample class //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
/*
Sample::Sample() : samplePointer(NULL) {
}

//////////////////////////////////////////////////////////////////////

Sample::~Sample() {
  printf("deleting Sample (%x)\n", this);
  deallocate();
}

//////////////////////////////////////////////////////////////////////

void Sample::deallocate(void) {
  if (samplePointer) FSOUND_Sample_Free(samplePointer);
  samplePointer = NULL;
}

//////////////////////////////////////////////////////////////////////

// Allocates an empty sample
void Sample::allocate(int numSamples, unsigned int mode, 
                      int defaultFrequency, int defaultVolume,
                      int defaultPan, int defaultPriority) {
  deallocate();
  samplePointer = FSOUND_Sample_Alloc(FSOUND_FREE, numSamples, mode, defaultFrequency, defaultVolume, defaultPan, defaultPriority);
}

//////////////////////////////////////////////////////////////////////

// Loads a sample from a file and allocates space for it
void Sample::load(const char * filename, unsigned int mode) {
  deallocate();
  samplePointer = FSOUND_Sample_Load(FSOUND_FREE, filename, mode, 0, 0);
}

//////////////////////////////////////////////////////////////////////

void Sample::upload(void * source, unsigned int mode) {
  if (samplePointer) FSOUND_Sample_Upload(samplePointer, source, mode);
}

//////////////////////////////////////////////////////////////////////

const char * Sample::getName(void) {
  if (samplePointer)
    return FSOUND_Sample_GetName(samplePointer);
  else
    return NULL;
}

//////////////////////////////////////////////////////////////////////

// Returns the length in samples
unsigned int Sample::getLength(void) {
  if (samplePointer)
    return FSOUND_Sample_GetLength(samplePointer);
  else
    return 0;
}
*/
//////////////////////////////////////////////////////////////////////
// Channel class /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

Channel::Channel(const char * filename, unsigned int mode)
//: ReferenceCounted()
{
  sample = FSOUND_Sample_Load(FSOUND_FREE, filename, mode, 0, 0);
  channel = -1;
  channel = FSOUND_PlaySoundEx(FSOUND_FREE, sample, NULL, TRUE);
}

//////////////////////////////////////////////////////////////////////

Channel::~Channel(void) {
  printf("deleting Channel (%x)\n", this);
  if (channel > -1) FSOUND_StopSound(channel);
  FSOUND_Sample_Free(sample);
}

//////////////////////////////////////////////////////////////////////

void Channel::start(void) {
  if (channel >= 0) {
    stop();
    //pause();
    //setPosition(0);
    //unpause();
  } //else {
    channel = FSOUND_PlaySound(FSOUND_FREE, sample);
  //}
}

//////////////////////////////////////////////////////////////////////

void Channel::stop(void) {
  if (channel >= 0) {
    FSOUND_StopSound(channel);
    channel = -1;
    //pause();
    //setPosition(0);
  }
}

//////////////////////////////////////////////////////////////////////

void Channel::pause(void) {
  FSOUND_SetPaused(channel, TRUE);
}

void Channel::unpause(void) {
  FSOUND_SetPaused(channel, FALSE);
}

//////////////////////////////////////////////////////////////////////

bool Channel::isPlaying(void) {
  if (channel >= 0) {
    //return (FSOUND_GetCurrentPosition(channel) < FSOUND_Sample_GetLength(sample));
    return FSOUND_IsPlaying(channel) == TRUE;
  } else
    return false;
}

//////////////////////////////////////////////////////////////////////

bool Channel::isPaused(void) {
  if (channel >= 0)
    return FSOUND_GetPaused(channel) == TRUE;
  else
    return true;
}

//////////////////////////////////////////////////////////////////////

void Channel::setLooping(bool enabled) {
  if (channel >= 0)
    FSOUND_SetLoopMode(channel, enabled ? FSOUND_LOOP_NORMAL : FSOUND_LOOP_OFF);
}

//////////////////////////////////////////////////////////////////////

void Channel::setPosition(unsigned int position) {
  if (channel >= 0)
    FSOUND_SetCurrentPosition(channel, position);
}

//////////////////////////////////////////////////////////////////////

void Channel::setFrequency(int frequency) {
  if (channel >= 0) FSOUND_SetFrequency(channel, frequency);
}

//////////////////////////////////////////////////////////////////////

void Channel::setVolume(int volume) {
  if (channel >= 0) FSOUND_SetVolume(channel, volume);
}

//////////////////////////////////////////////////////////////////////

bool Channel::getLooping(void) {
  if (channel >= 0)
    return FSOUND_GetLoopMode(channel) != FSOUND_LOOP_OFF;
  else
    return false;
}

//////////////////////////////////////////////////////////////////////

unsigned int Channel::getPosition(void) {
  if (channel >= 0)
    return FSOUND_GetCurrentPosition(channel);
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////

int Channel::getFrequency(void) {
  if (channel >= 0)
    return FSOUND_GetFrequency(channel);
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////

int Channel::getVolume(void) {
  if (channel >= 0)
    return FSOUND_GetVolume(channel);
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// Set the position (in meters)
void Channel3D::setPosition(float x, float y, float z) {
  float pos[3];
  pos[0] = x;
  pos[1] = y;
  pos[2] = z;
  FSOUND_3D_SetAttributes(channel, pos, NULL);
  //FSOUND_3D_SetMinMaxDistance(channel, 0.1f, 1000000.0f);
}

//////////////////////////////////////////////////////////////////////

  // Set the velocity (in meters / second)
void Channel3D::setVelocity(float vx, float vy, float vz) {
  float vel[3];
  vel[0] = vx;
  vel[1] = vy;
  vel[2] = vz;
  FSOUND_3D_SetAttributes(channel, NULL, vel);
}

//////////////////////////////////////////////////////////////////////
// I/O wrapper around TVFSFileStream for FMOD ////////////////////////
//////////////////////////////////////////////////////////////////////

void * F_CALLBACKAPI userOpen(const char * name) {
  return vfsFileOpen(name, vfsOpenRead|vfsShareDenyNone);
}

//////////////////////////////////////////////////////////////////////

int F_CALLBACKAPI userRead(void * buffer, int size, void * handle) {
  TVFSStream * stream = (TVFSStream *)handle;
  return stream->Read(buffer, size);
}

//////////////////////////////////////////////////////////////////////

void F_CALLBACKAPI userClose(void * handle) {
  TVFSStream * stream = (TVFSStream *)handle;
  delete stream;
}

//////////////////////////////////////////////////////////////////////

int F_CALLBACKAPI userSeek(void * handle, int pos, signed char mode) {
  TVFSStream * stream = (TVFSStream *)handle;
  if (mode == SEEK_SET)
    stream->Seek(pos, vfsFromBeginning);
  else if (mode == SEEK_CUR)
    stream->Seek(pos, vfsFromCurrent);
  else if (mode == SEEK_END)
    stream->Seek(pos, vfsFromEnd);
  else
    return -1;
  return 0;
}

//////////////////////////////////////////////////////////////////////

int F_CALLBACKAPI userTell(void * handle) {
  TVFSStream * stream = (TVFSStream *)handle;
  return stream->getPosition();
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

void soundInit(void) {
  // drops into software if card supports fewer than this
  FSOUND_SetMinHardwareChannels(16);

  // Init
  FSOUND_SetOutput(FSOUND_OUTPUT_DSOUND);
  FSOUND_SetSpeakerMode(FSOUND_SPEAKERMODE_SURROUND);
  FSOUND_Init(44100, 32, FSOUND_INIT_GLOBALFOCUS | FSOUND_INIT_DSOUND_HRTF_FULL);
  //FSOUND_3D_SetRolloffFactor(1.0);

  // Setup file callbacks
  FSOUND_File_SetCallbacks(userOpen, userClose, userRead, userSeek, userTell);
}

//////////////////////////////////////////////////////////////////////

void soundCleanup(void) {
  FSOUND_Close();
}

//////////////////////////////////////////////////////////////////////

// Fixme: should do some form of duplicate-load detection!
Channel * soundLoad(const char * filename) {
  if (vfsFileExists(filename)) {
    Channel * result = new Channel(filename);
//    result->addReference();
    return result;
  } else {
    printf("failed to load %s\n", filename);
    return NULL;
  }
}

//////////////////////////////////////////////////////////////////////

void soundMoveListener(float * position, float * velocity, float * forward, float * up) {
  FSOUND_3D_Listener_SetAttributes(position, velocity,
                                   forward[0], forward[1], forward[2],
                                   up[0], up[1], up[2]);
 FSOUND_Update();
}

//////////////////////////////////////////////////////////////////////
