//////////////////////////////////////////////////////////////////////
// sound.h - Sound wrapper ///////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#ifndef SOUND_INCLUDE
#define SOUND_INCLUDE

//////////////////////////////////////////////////////////////////////
// Includes //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>

#include "system.h"
#include <fmod.h>
#include <fmod_errors.h>

//////////////////////////////////////////////////////////////////////
/*
class Sample {
private:
  FSOUND_SAMPLE * samplePointer;
public:
  Sample();
  ~Sample();

  void deallocate(void);
  void allocate(int numSamples, unsigned int mode, int defaultFrequency, int defaultVolume, int defaultPan, int defaultPriority);
  void load(const char * filename, unsigned int mode);
  void upload(void * source, unsigned int mode);
  unsigned int getLength(void);
  const char * getName(void);
};
*/
//////////////////////////////////////////////////////////////////////
// Channel class /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

class Channel/* : public ReferenceCounted */{
public:
  Channel(const char * filename, unsigned int mode = FSOUND_NORMAL);
  ~Channel();
  void start(void);
  void stop(void);
  void pause(void);
  void unpause(void);

  bool isPlaying(void);
  bool isPaused(void);

  void setLooping(bool enabled);
  void setPosition(unsigned int position);
  void setFrequency(int frequency);
  void setVolume(int volume);

  bool getLooping(void);
  unsigned int getPosition(void);
  int getFrequency(void);
  int getVolume(void);
protected:
  FSOUND_SAMPLE * sample;
  int channel;
};

//////////////////////////////////////////////////////////////////////

class Channel3D : public Channel {
protected:
public:
  Channel3D(const char * filename, int flags = 0)
    : Channel(filename, FSOUND_HW3D | flags)
  {
    setLooping(true);
    setVolume(64);
  }

  // Set the position (in meters)
  void setPosition(float x, float y, float z);

  // Set the velocity (in meters / second)
  void setVelocity(float vx, float vy, float vz);
};

//////////////////////////////////////////////////////////////////////

class Song {
private:
  FMUSIC_MODULE * moduleHandle;
public:
  Song()  { moduleHandle = NULL; }
  ~Song() { 
    printf("deleting Song (%x)\n", this);
    FMUSIC_FreeSong(moduleHandle);
  }

  void load(const char * filename) { moduleHandle = FMUSIC_LoadSong(filename); }

  void start()   { FMUSIC_PlaySong(moduleHandle); }
  void stop()    { FMUSIC_StopSong(moduleHandle); }
  void pause()   { FMUSIC_SetPaused(moduleHandle, TRUE); }
  void unpause() { FMUSIC_SetPaused(moduleHandle, FALSE); }

  void setSpeed(float speed)     { FMUSIC_SetMasterSpeed(moduleHandle, speed); }
  void setVolume(int volume)     { FMUSIC_SetMasterVolume(moduleHandle, volume); }
  void setLooping(bool looping)  { FMUSIC_SetLooping(moduleHandle, looping); }
  void setSeperation(float dist) { FMUSIC_SetPanSeperation(moduleHandle, dist); }

  bool isPaused()     { return FMUSIC_GetPaused(moduleHandle) == TRUE; }
  bool isPlaying()    { return FMUSIC_IsPlaying(moduleHandle) == TRUE; }
  bool isFinished()   { return FMUSIC_IsFinished(moduleHandle) == TRUE; }
  int getSpeed(void)  { return FMUSIC_GetSpeed(moduleHandle); }
  int getVolume(void) { return FMUSIC_GetMasterVolume(moduleHandle); }
};

//////////////////////////////////////////////////////////////////////

void soundInit(void);
void soundCleanup(void);

// position in meters
// velocity in meters/second
// forward is forward vector of listener reference frame
// up is up vector of listener reference frame
void soundMoveListener(float * position, float * velocity, float * forward, float * up);

Channel * soundLoad(const char * filename);

//////////////////////////////////////////////////////////////////////

#endif