//////////////////////////////////////////////////////////////////////

#include "system.h"

#ifdef _WIN32
#include "Mmsystem.h"
#endif

//////////////////////////////////////////////////////////////////////

float masterTick;
int screenWidth;
int screenHeight;

//////////////////////////////////////////////////////////////////////

TJObject::TJObject(void) {
  FValid = false;
}

//////////////////////////////////////////////////////////////////////

TJObject::~TJObject(void) {
  release();
  FValid = false;
}

//////////////////////////////////////////////////////////////////////

void TJObject::release(void) {
  FValid = false;
}

//////////////////////////////////////////////////////////////////////

bool TJObject::valid(void) const {
  return FValid;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

uint32 sysGetTime(void) {
  return timeGetTime();
}

//////////////////////////////////////////////////////////////////////

uint32 sysTimeElapsed(uint32 * lastTime) {
  uint32 curTime = sysGetTime();
  uint32 result = curTime - *lastTime;
  *lastTime = curTime;
  return result;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

void SetColor(float * color, float r, float g, float b, float a) {
  color[0] = r;
  color[1] = g;
  color[2] = b;
  color[3] = a;
}

//////////////////////////////////////////////////////////////////////

void SetVector4(float * pos, float x, float y, float z, float w) {
  pos[0] = x;
  pos[1] = y;
  pos[2] = z;
  pos[3] = w;
}

//////////////////////////////////////////////////////////////////////

void SetVector3(float * pos, float x, float y, float z) {
  pos[0] = x;
  pos[1] = y;
  pos[2] = z;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// Returns local year/month/day and time zone (hours from GMT)
// i.e. central time is -6
void sysGetDate(int * year, int * month, int * day, int * tz) {
  SYSTEMTIME st;
  TIME_ZONE_INFORMATION tzi;
  GetLocalTime(&st);
  GetTimeZoneInformation(&tzi);

  *tz = -tzi.Bias / 60; // bias is in terms of minutes
  *year = st.wYear;
  *month = st.wMonth;
  *day = st.wDay;
}

//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////

void j_fillMemory(void *dest, uint32 count, uint8 c) {
  uint32 size32 = count>>2;
  uint32 fill = (c<<24|c<<16|c<<8|c);
  uint32 * dest32 = (uint32*)dest;

  switch ((count-(size32<<2))) {
    case 3: ((uint8*)dest)[count - 3] = c;
    case 2: ((uint8*)dest)[count - 2] = c;
    case 1: ((uint8*)dest)[count - 1] = c;
  }

  while (size32-- > 0)
    *(dest32++) = fill;
}

//////////////////////////////////////////////////////////////////////

void j_copyMemory(void *dest, const void *src, uint32 count) {
  uint32 size32 = count >> 2;
  uint32 * dest32 = (uint32*)dest;
  uint32 * src32 = (uint32*)src;

  switch ((count-(size32<<2))) {
    case 3: ((uint8*)dest)[count-3] = ((uint8*)src)[count-3];
    case 2: ((uint8*)dest)[count-2] = ((uint8*)src)[count-2];
    case 1: ((uint8*)dest)[count-1] = ((uint8*)src)[count-1];
  }

  /* Modified [05-01-03] to use forward caching */
  while (size32-- > 0)
    *(dest32++) = *(src32++);
}

//////////////////////////////////////////////////////////////////////

void j_moveMemory(void *dest, const void *src, uint32 count) {
  uint32 size32 = count >> 2;
  uint32 i;
  uint32 * dest32 = (uint32*)dest;
  uint32 * src32 = (uint32*)src;

  if( dest > src ) {
    switch( (count-(size32<<2)) ) {
    case 3:  ((uint8*)dest)[count-1] = 
      ((uint8*)src)[count-1];
      ((uint8*)dest)[count-2] = 
      ((uint8*)src)[count-2];
      ((uint8*)dest)[count-3] = 
      ((uint8*)src)[count-3];
      break;
    case 2:  ((uint8*)dest)[count-1] = 
      ((uint8*)src)[count-1];
      ((uint8*)dest)[count-2] = 
      ((uint8*)src)[count-2];
      break;
    case 1:  ((uint8*)dest)[count-1] = 
      ((uint8*)src)[count-1];
      break;
    }

    while (size32-- > 0)
      dest32[size32] = src32[size32];
  } else {
    for (i = 0; i < size32; i++)
      *(dest32++) = *(src32++);

    switch ((count-(size32<<2))) {
      case 3: *(dest32++) = *(src32++);
      case 2: *(dest32++) = *(src32++);
      case 1: *(dest32++) = *(src32++);
    }
  }
}

//////////////////////////////////////////////////////////////////////
