//////////////////////////////////////////////////////////////////////

#ifndef SYSTEM_INCLUDE
#define SYSTEM_INCLUDE

//////////////////////////////////////////////////////////////////////

//#define _WIN32_WINDOWS 0x0410
//#define WINVER 0x0400
//#define _WIN32_WINNT 0x0400

// Put windows on a diet
#define WIN32_LEAN_AND_MEAN 1

#include <windows.h>
#include <GL\GL.h>
#include <GL\glu.h>
#include <stdio.h>
#include <list>

#pragma pack(push, 1)

//////////////////////////////////////////////////////////////////////

// Disable the 64->32 and 32->64 typecast thingies
// The amount of code that will break going to native 64-bits will be
// INSANE!!!,  not only my code, but everywhere
// I'm also disabling double->float conversions, because they're
// bothering me, can easily be re-enabled later
#ifdef _MSC_VER
#pragma warning ( disable : 4311 4312 4244)
#endif

//////////////////////////////////////////////////////////////////////
// If these end up getting used a whole bunch, an inline opcode would
// be far better, bswap here or the equiv. on those forsaken procs.
//////////////////////////////////////////////////////////////////////

// A lilliputan am I, for the little end truly is superior
#define localizeLEQuad(a) (a)
#define localizeLEPair(a) (a)

// May motorolla suffer everlasting torture!
#define localizeBEQuad(a) ((((a)&0xFF)<<24) + (((a)&0xFF00)<<8) + (((a)&0xFF0000)>>8) + ((a)>>24))
#define localizeBEPair(a) ((((a)&0xFF)<<16) + ((a)>>8))

//////////////////////////////////////////////////////////////////////

typedef signed char int8, *pint8;
typedef signed short int16, *pint16;
typedef signed int int32, *pint32;
typedef signed __int64 int64, *pint64;
typedef unsigned char uint8, *puint8;
typedef unsigned short uint16, *puint16;
typedef unsigned int uint32, *puint32;
typedef unsigned __int64 uint64, *puint64;

typedef float vec2_t[2];
typedef float vec3_t[3];
typedef float vec4_t[4];
typedef int16 point3s[3];
typedef unsigned char byte;

typedef struct {
  float mins[3];
  float maxs[3];
} TBBox;

typedef float TColor[4];

//////////////////////////////////////////////////////////////////////

template<class T> class TMMPointer {
protected:
   T* obj;
public:
   //Constructors, destructor, assignments and accessors same as before
   //Constructors - basic
   TMMPointer()
   {
      obj = NULL;
   }
   //Constructing with a pointer
   TMMPointer(T *o)
   {
      obj = NULL;
      *this = NULL;
   }
   //Constructing with another smart pointer (copy constructor)
   TMMPointer(const TMMPointer<T> &p)
   {
      obj = NULL;
      *this = p;
   }

   //Destructor
   ~TMMPointer()
   {
      if (obj) obj->Release();
   }

   // Assignement operators - assigning a plain pointer
   inline operator =(T *o)
   {
      if (obj) obj->Release();
      obj = o;
      if (obj) obj->AddRef();
   }

   // Assigning another smart pointer
   inline operator =(const TMMPointer<T> &p)
   {
      if (obj) obj->Release();
      obj = p.obj;
      if (obj) obj->AddRef();
   }

   //Access as a reference
   inline T& operator *() const
   {
      //assert(obj!=0 && "Tried to * on a NULL smart pointer");
      return *obj;
   }
   //Access as a pointer
   inline T* operator ->() const
   {
      //assert(obj!=0 && "Tried to -> on a NULL smart pointer");
      return obj;
   }

   //Conversion - allow the smart pointer to be automatically converted to type T*
   inline operator T*() const
   {
      return obj;
   }

   inline bool isValid() const {
     return (obj != NULL);
   }

   inline bool operator !() {
      return !(obj);
   }

   inline bool operator ==(const TMMPointer<T> &p) const
   {
      return (obj==p.obj);
   }

   inline bool operator ==(const T* o) const
   {
      return (obj==o);
   }
};

/*class ReferenceCounted {
private:
  unsigned int myReferenceCount;
public:
  ReferenceCounted() : myReferenceCount(0) {}

  void addReference(void) {
    myReferenceCount++;
  }

  void removeReference(void) {
    myReferenceCount--;
    if (myReferenceCount == 0) delete this;
  }
};*/

class TJObject {
public:
  TJObject(void);
  ~TJObject(void);
  bool valid(void) const;
  // bool initialize(stuff), returns FValid
  virtual void release(void);
  
  static void collectGarbage(void);
  void addReference(void) { referenceCount++; }
  void removeReference(void) { referenceCount--; }
protected:
  bool FValid;
private:
  static std::list<TJObject *> liveObjects;
  uint32 referenceCount;
};

typedef TMMPointer<TJObject *> TJObjectPtr;

//////////////////////////////////////////////////////////////////////

#ifndef __DATE__
#define __DATE__ "DateErr"
#endif

#ifndef __FILE__
#define __FILE__ "FileErr"
#endif

#ifndef __LINE__
#define __LINE__ -1
#endif

#define ASSERT_VALID(a) \
  if ((this == NULL) || (!FValid)) { \
    j_printf("ERROR: "##a##" called on an uninitialized object!\n"); \
    return; \
  }

#define ASSERT_VALID_RET(a, b) \
  if ((this == NULL) || (!FValid)) { \
    j_printf("ERROR: "##a##" called on an uninitialized object!\n"); \
    return (b); \
  }

extern float masterTick;
extern int screenWidth;
extern int screenHeight;

//////////////////////////////////////////////////////////////////////

uint32 sysGetTime(void);
uint32 sysTimeElapsed(uint32 * lastTime);

// Returns local year/month/day and time zone (hours from GMT)
// i.e. central time is -6
void sysGetDate(int * year, int * month, int * day, int * tz);

//////////////////////////////////////////////////////////////////////

void SetColor(float * color, float r, float g, float b, float a);
void SetVector4(float * pos, float x, float y, float z, float w);
void SetVector3(float * pos, float x, float y, float z);

//////////////////////////////////////////////////////////////////////

#define j_printf printf

//////////////////////////////////////////////////////////////////////

void j_fillMemory(void *dest, uint32 count, uint8 c);

#define j_zeroMemory(dest,count) j_fillMemory(dest,count,0)

void j_copyMemory(void *dest, const void *src, uint32 count);
void j_moveMemory(void *dest, const void *src, uint32 count);

//////////////////////////////////////////////////////////////////////

#endif

//////////////////////////////////////////////////////////////////////