//////////////////////////////////////////////////////////////////////
// Includes //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#include "system.h"
#include "jstring.h"

#include "textures.h"

//#include "loadPCX.h"
//#include "loadWAL.h"
#include "loadTGA.h"
//#include "loadJPG.h"

//////////////////////////////////////////////////////////////////////
// TTexture //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

/*
Note: loadAsset will never return null (unless the system runs out of
      memory, which is a catostrophic failure in any event)
      If the specified asset could not be loaded, it will instead return
      an uninitialized asset (asset->valid() returns false), which will
      fail on any call to the asset (runtime silent, writes to debug log).
      This way, missing resources can be logged without crashing the game,
      and it means the error checking is localized to the object, everyone
      else can just assume that say, a texture is always loaded, no putting
      error checking code in ten million different freaking places!

      TAssets are also reference counted, they should never be delete-ed
      by users, always call releaseAsset(asset) instead!
*/
#if 0
TAsset * loadAsset(filename, type);

// Release an asset (functionally identical to asset->release())
void releaseAsset(TAsset * asset);

class TAsset : public TJObject {
protected:
  int myReferenceCount;
  friend releaseAsset(TAsset * asset);
};

//////////////////////////////////////////////////////////////////////

TAsset * loadAsset(const char * filename, int type) {
  // First, look for a pre-loaded asset
  TAsset * result = findAsset(filename);
  if (result != NULL) {
    result->myReferenceCount++;
    return result;
  }

  // Not loaded, we need to find it
  if (type == 0) type = autodetectType(filename);

  switch (type) {
    case 0:
      // texture
    default:
      j_printf("Fixme: in loadAsset(), never got around to writing this bit!\n");
  }

  return result;
}

total types of assets I easily forsee:
  textures
  shaders
  music
  sound effects
  lut (arbitrary data file, just a wrapper around a VFS file)


  write code like:
void quakeLevel::load:
  for each texture do handle = loadTextureAsset(texture)
always returns a TTexture *

this will always return the correct type of asset

#endif

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

TAssetFactory::~TAssetFactory(void) {
  // fixme: implement
}

//////////////////////////////////////////////////////////////////////

TAssetFactory::TAssetFactory(int size) {
  maxNumAssets = size;
  assets = new PAsset[maxNumAssets];

  for (int i = 0; i < maxNumAssets; i++) assets[i] = NULL;   
}

//////////////////////////////////////////////////////////////////////

TTexture * TAssetFactory::findAsset(const char * filename) {
  return NULL;
}

//////////////////////////////////////////////////////////////////////

void TAssetFactory::addAsset(TTexture * asset, const char * name) {
  int i, j;

  for (i = 0; i < maxNumAssets; i++) {
    if (assets[i] == NULL) break;
  }
  
  // Check to see if we need to resize the array (expensive)
  if (i == maxNumAssets) {
    int newMax = (3*maxNumAssets)>>1;
    TAsset ** newAssets = new PAsset[newMax];
    for (j = 0; j < maxNumAssets; j++) newAssets[j] = assets[j];
    for (j = maxNumAssets; j < newMax; j++) newAssets[j] = NULL;
    i = maxNumAssets;  // were we're heading

    // Swap teh buffers and release
    maxNumAssets = newMax;
    delete assets;
    assets = newAssets;
  }

  // Copy the name and create the entry
  j_stringFill(assets[i]->name, 0, MAX_ASSET_NAME_LENGTH);
  j_stringCopy(name, assets[i]->name, MAX_ASSET_NAME_LENGTH-1);
}

//////////////////////////////////////////////////////////////////////

void TAssetFactory::deleteAsset(TTexture * asset) {
  for (int i = 0; i < maxNumAssets; i++) {
    if (assets[i] == asset) assets[i] = NULL;
  }

  delete asset;
}

//////////////////////////////////////////////////////////////////////

TTexture * TAssetFactory::loadAsset(const char * filename) {
  TTexture * result;

  // See if we have already loaded the asset
  result = findAsset(filename);
  if (result == NULL) {
    // Nope, so we should load the asset
    result = new TTexture();
    //fixme: result->initialize(filename);
    addAsset(result, filename);
  }

  // Add a reference to the asset and return
  result->acquireAsset();
  return result;
}

//////////////////////////////////////////////////////////////////////

// Release an asset (functionally identical to asset->release())
void TAssetFactory::releaseAsset(TAsset * asset) {
}


//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

// Returns true if the asset is loaded
bool TAsset::acquireAsset(void) {
  myReferenceCount++;
  return FValid;
}

//////////////////////////////////////////////////////////////////////

// Return true if the asset should be deleted
bool TAsset::releaseAsset(void) {
  myReferenceCount--;
  if (myReferenceCount <= 0) {
    release();
    return true;
  }
  return false;
}


//////////////////////////////////////////////////////////////////////
// activate - activate the encapsulated texture //////////////////////
//////////////////////////////////////////////////////////////////////

void TTexture::activate(void) {
  ASSERT_VALID("TTexture::activate")
  glBindTexture(GL_TEXTURE_2D, texID);
}

//////////////////////////////////////////////////////////////////////

int TTexture::getWidth(void) {
  if (!FValid) return 0;
  return myWidth;
}

//////////////////////////////////////////////////////////////////////

int TTexture::getHeight(void) {
  if (!FValid) return 0;
  return myHeight;
}

//////////////////////////////////////////////////////////////////////

bool TTexture::initialize(int width, int height, int bpp) {
  myWidth = width;
  myHeight = height;
  myBPP = bpp;

  // Allocate a buffer
  myDataSize = width*height*(bpp>>3);
  myData = new byte[myDataSize];
  if (myData == NULL) {
    // fixme: Doh, out of memory
    myDataSize = 0;
    return false;
  }

  // Generate a texture handle
  glGenTextures(1, &texID);
  FValid = true;
  return FValid;
}

//////////////////////////////////////////////////////////////////////

void TTexture::release(void) {
  if (!FValid) return;

  // Free the data buffer if allocated
  if (myDataSize > 0) {
    delete [] myData;
    myDataSize = 0;
  }

  // Delete the texture handle
  glDeleteTextures(1, &texID);

  // No longer valid, obviously
  FValid = false;
}

//////////////////////////////////////////////////////////////////////

void TTexture::upload(void) {
  ASSERT_VALID("TTexture::upload")

  // Bind the data to an OpenGL texture handle
  glBindTexture(GL_TEXTURE_2D, texID);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  GLenum type = (myBPP == 24) ? GL_RGB : GL_RGBA;
  //glTexImage2D(GL_TEXTURE_2D, 0, type, myWidth, myHeight,
//    0, type, GL_UNSIGNED_BYTE, myData);
  
  gluBuild2DMipmaps(GL_TEXTURE_2D, myBPP>>3, myWidth, myHeight, type, GL_UNSIGNED_BYTE, myData);
  // Return successfully
  FValid = true;
}

//////////////////////////////////////////////////////////////////////

byte * TTexture::getBuffer(void) {
  return myData;
}

//////////////////////////////////////////////////////////////////////

void TTexture::freeBuffer(void) {
  if (myDataSize > 0) {
    delete [] myData;
    myDataSize = 0;
    myData = NULL;
  }
}

//////////////////////////////////////////////////////////////////////
// TTextureList //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

TTextureList* TTextureList::Top = NULL;
int TTextureList::UniqueIndex = 1;

//////////////////////////////////////////////////////////////////////

TTextureList::TTextureList () {
   Texture = NULL;
   Next = NULL;
   strcpy(myFilename, "\0\0\0\0");
}

//////////////////////////////////////////////////////////////////////
// Destructor - Delete this object and all following objects /////////
//////////////////////////////////////////////////////////////////////

TTextureList::~TTextureList() {
  if (Next != NULL) Delete ();
}

//////////////////////////////////////////////////////////////////////
// Delete - Delete this object and the rest in the list //////////////
//////////////////////////////////////////////////////////////////////

void TTextureList::Delete(void) {
   if (Next != NULL) delete Next;
   Next = NULL;
}

//////////////////////////////////////////////////////////////////////
// Loader - Load a Texture into memory, once /////////////////////////
//////////////////////////////////////////////////////////////////////

TTexture* TTextureList::Loader(const char* filename) {
  return Loader(filename, true);
}

TTexture* TTextureList::Loader(const char* filename, bool makeMips) {
  TTexture* retTexture = NULL;

  //fixme if (DEBUG) j_printf ("Attempting to load %s\n", filename);
  if (Top == NULL)
    retTexture = AddTexture(filename);
  else {
    retTexture = Top->Search(filename);
    if (retTexture == NULL) retTexture = AddTexture(filename);
  }

  //fixme if (DEBUG) j_printf ("Loaded Texture to memory %p\n", retTexture);

  return retTexture;
}

//////////////////////////////////////////////////////////////////////

TTexture* TTextureList::CreateUnique(const char* filename) {
  int len = j_stringLength(filename);
  char * buffer;

  // If the user didn't specify a name, throw on a number
  if (!len) {
    // fixme: for a multitude of reasons
    len = 1;
    // wrong length calculation, this is soooo lame
    for (int temp = UniqueIndex; temp > 0; temp = temp >> 4) len += 2;
    buffer = new char[len + 2];
    buffer[0] = '@';
    int i = 1;
    // really really poor mans hex->ascii
    for (int temp = UniqueIndex; temp > 0; temp = temp >> 4) {
      buffer[i++] = 'A' + temp & 0xF;
      len += 2;
    }
    buffer[i] = 0;

    UniqueIndex++;
  } else {
    len++;
    buffer = new char[len + 1];
    strcpy(&(buffer[1]), filename);
    buffer[0] = '@';
  }
  TTexture * result = AddTexture(buffer);
  delete [] buffer;

  return result;
}

//////////////////////////////////////////////////////////////////////
// AddTexture - Add a Texture to the list of textures ////////////////
//////////////////////////////////////////////////////////////////////

TTexture* TTextureList::AddTexture(const char* filename) {
  TTextureList* lastTextureList = Top;

  if (lastTextureList != NULL) {
    while (lastTextureList->Next != NULL) lastTextureList = lastTextureList->Next;
    lastTextureList->Next = new TTextureList ();
    lastTextureList = lastTextureList->Next;
  } else {
    Top = new TTextureList();
    lastTextureList = Top;
  }
  strcpy(lastTextureList->myFilename, filename);

  // We're done if this is a procedural texture
  if (filename[0] == '@') {
    lastTextureList->Texture = new TTexture();
    return lastTextureList->Texture;
  }

  // Choose an appropriate loader from the various types supported
  // Should support .tga, .wal, .jpg, .png, .pcx
  TVFSStream * stream = vfsFileOpen(filename, vfsOpenRead | vfsShareDenyNone);
  if (stream == NULL) {
    j_printf("Error: Could not open image file %s\n", filename);
    return NULL;
  }

  if (matchExtension(filename, ".tga")) {
    TTGATexture * tga = new TTGATexture();
    tga->loadTGA(stream, filename);
    lastTextureList->Texture = tga;
/*
  } else if (matchExtension(filename, ".wal")) {
    TWALTexture * wal = new TWALTexture();
    wal->loadWAL(stream, filename);
    lastTextureList->Texture = wal;
  } else if (matchExtension(filename, ".jpg")) {
    TJPGTexture * jpg = new TJPGTexture();
    jpg->loadJPG(stream, filename);
    lastTextureList->Texture = jpg;
  } else if (matchExtension(filename, ".pcx")) {
    TPCXTexture * pcx = new TPCXTexture();
    pcx->loadPCX(stream, filename);
    lastTextureList->Texture = pcx;
*/
  }
//  delete stream;

  return lastTextureList->Texture;
}

//////////////////////////////////////////////////////////////////////
// Search - Search the list for a loaded Texture /////////////////////
//////////////////////////////////////////////////////////////////////

TTexture* TTextureList::Search(const char *filename) {
  if (!j_stringCompare(filename, myFilename, false, 0))
    return Texture;
  else if (Next != NULL)
    return Next->Search(filename);
  else
    return NULL;
}

//////////////////////////////////////////////////////////////////////
