//////////////////////////////////////////////////////////////////////
// textures.h - Interface for the TTexture class /////////////////////
//////////////////////////////////////////////////////////////////////

#ifndef TEXTURES_H
#define TEXTURES_H

//////////////////////////////////////////////////////////////////////

#include "system.h"
#include "jfilesystem.h"

//////////////////////////////////////////////////////////////////////

#define MAX_ASSET_NAME_LENGTH 512

//////////////////////////////////////////////////////////////////////
// TAsset ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

class TAsset : public TJObject {
protected:
  int myReferenceCount;
public:
  char name[MAX_ASSET_NAME_LENGTH];
  bool acquireAsset(void);
  bool releaseAsset(void);
};

typedef TAsset * PAsset;

//////////////////////////////////////////////////////////////////////
// TTexture //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

class TTexture : public TAsset {
public:
  virtual void release(void);

  void activate(void);
//  bool initialize(TVFSStream * stream, const char * filename);
  //virtual void release(void) {};
  //virtual ~TTexture();
  int getWidth(void);
  int getHeight(void);
  void upload(void);
  void freeBuffer(void);
  byte * getBuffer(void);

  bool initialize(int width, int height, int bpp);
protected:
  byte * myData;
  int myWidth, myHeight;
  int myDataSize;
  int myBPP;
  GLuint texID;
};

//////////////////////////////////////////////////////////////////////

typedef TTexture * PTexture;

//////////////////////////////////////////////////////////////////////


class TAssetFactory {
protected:
  int maxNumAssets;
  TAsset ** assets;

  void addAsset(TTexture * asset, const char * name);
  void deleteAsset(TTexture * asset);
public:
  TAssetFactory(int size);
  ~TAssetFactory(void);
  TTexture * findAsset(const char * filename);
  TTexture * loadAsset(const char * filename);
  void releaseAsset(TAsset * asset);
};

//////////////////////////////////////////////////////////////////////
// TTextureList //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

class TTextureList {
public:
  static TTexture* Loader(const char* filename);
  static TTexture* Loader(const char* filename, bool makeMips);
  static TTexture* CreateUnique(const char* filename);
  TTextureList();
  virtual ~TTextureList();
private:
  TTexture* Search(const char* filename);
  static TTexture* AddTexture(const char* filename);
  void Delete();
  static TTextureList* Top;
  static int UniqueIndex;
  TTexture* Texture;
  TTextureList* Next;
  char myFilename[160];
};

//////////////////////////////////////////////////////////////////////

#endif

