//////////////////////////////////////////////////////////////////////

#include "system.h"
#include "jfilesystem.h"
#include "vfsPakFile.h"
#include "vfsWin32.h"

//////////////////////////////////////////////////////////////////////
// TVFSMountPak //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#define PACK_MAGIC (*(uint32*)"PACK")

TVFSMountPak::TVFSMountPak(const char * root) {
  FValid = false;

  // Read in the header
  if (!FileExists(root)) return;
  FRoot = new TVFSStreamWin32(root, vfsOpenRead | vfsShareDenyWrite);
  FRoot->Read(&header, sizeof(TPakHeader));
  localizeLEQuad(header.DirOffset);
  localizeLEQuad(header.DirSize);

  // Make sure root is a valid quake .pak file
  if (header.MagicNum == PACK_MAGIC) {
    // Read in the directory
    numFiles = header.DirSize / sizeof(TPakEntry);
    directory = new TPakEntry[header.DirSize];
    FRoot->Seek(header.DirOffset, vfsFromBeginning);
    FRoot->Read(directory, header.DirSize);

    // Correct the endianness of the directory entries if needed
    for (int i = 0; i < numFiles; i++) {
      localizeLEQuad(directory[i].Offset);
      localizeLEQuad(directory[i].Size);
    }

    FValid = true;
  } else {
    // raise an error, this is not a valid Quake .pak file
    delete root;
  }
}

//////////////////////////////////////////////////////////////////////

TVFSMountPak::~TVFSMountPak(void) {
  if (FValid) {
    delete [] directory;
    delete FRoot;
  }
}

//////////////////////////////////////////////////////////////////////

bool TVFSMountPak::fsFileExists(const char * FileName) {
  return getDirectoryIndex(FileName) > -1;
}

//////////////////////////////////////////////////////////////////////

TVFSStream * TVFSMountPak::fsFileOpen(const char * FileName, TVFSFileMode Mode) {
  int index = getDirectoryIndex(FileName);
  if (index > -1) 
    return new TVFSStreamPak(&(directory[index]), FRoot);
  return NULL;
}

//////////////////////////////////////////////////////////////////////

void TVFSMountPak::fsListFiles(const char * Path, TStringList * list) {
  int length;
  char * buffer;
  char name[2048];
  
  j_stringFill(name, 0, 2048);
  j_stringCopy(Path, name, 2047);

  // Correct the path delimiters
  vfsTranslatePath(name, '/');

  length = j_stringLength(name);
  for (int i = 0; i < numFiles; i++) {
    buffer = directory[i].FileName;

    // To be a match, buffer must match path for length(path) chars,
    // and the remaining characters (there must be 1+) shouldn't
    // contain a directory (i.e. with no / characters)
    if (j_stringLength(buffer) > length) {
      if ((length == 0) || (j_stringCompare(directory[i].FileName, name, false, 0) == 0)) {
        char * p;
        for (p = &(buffer[length]); (*p) && (*p != '/'); p++) ;
        if (!*p) if (list->IndexOf(buffer) < 0) list->Add(buffer);
      }
    }
  }
}

//////////////////////////////////////////////////////////////////////

int TVFSMountPak::getDirectoryIndex(const char * filename) {
  int length = j_stringLength(filename);
  char buffer[56];
  if (length < 56) {
    j_stringCopy(filename, buffer, 55);
    buffer[length] = 0;
    vfsTranslatePath(buffer, '/');
    for (int i = 0; i < numFiles; i++)
      if (!(j_stringCompare(directory[i].FileName, buffer, false, 0))) return i;
  }
  return -1;
}

//////////////////////////////////////////////////////////////////////
// TVFSStreamPak ////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

TVFSStreamPak::TVFSStreamPak(PPakEntry entry, TVFSStream * pak) {
  info = entry;
  mount = pak;
  FValid = true;
  FOffset = 0;
}

//////////////////////////////////////////////////////////////////////

TVFSStreamPak::~TVFSStreamPak(void) {}

//////////////////////////////////////////////////////////////////////

int TVFSStreamPak::Read(void * buffer, int count) {
  mount->Seek(info->Offset + FOffset, vfsFromBeginning);
  int Result = mount->Read(buffer, count);
  if (Result < 0) Result = 0;
  FOffset += Result;

  return Result;
}

//////////////////////////////////////////////////////////////////////

int TVFSStreamPak::Seek(int offset, TVFSOrigin origin) {
  switch (origin) {
    case vfsFromBeginning:
      FOffset = offset;
      break;
    case vfsFromEnd:
      FOffset = info->Size + offset;
      break;
    default:
      FOffset += offset;
  }

  return FOffset;
}

//////////////////////////////////////////////////////////////////////

bool TVFSStreamPak::setSize(int newSize) {
  return false;
}

//////////////////////////////////////////////////////////////////////

int TVFSStreamPak::Write(const void * buffer, int count) {
  return 0;
}

//////////////////////////////////////////////////////////////////////
