//////////////////////////////////////////////////////////////////////

#ifndef __vfs_pak_file__
#define __vfs_pak_file__

//////////////////////////////////////////////////////////////////////

#include "system.h"
#include "jfilesystem.h"
#include "vfsWin32.h"

//////////////////////////////////////////////////////////////////////
// Types /////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

typedef struct {
  uint32 MagicNum;    // Magic signature 'PACK'
  int32 DirOffset;    // Position of the directory in the file
  int32 DirSize;      // Number of entries * 64
} TPakHeader;

typedef struct {
  char FileName[56];   // Name of the file, unix style + extension, ASCIIZ
  int32 Offset;        // Position of the entry in the file
  int32 Size;          // Size of the entry
} TPakEntry, * PPakEntry;

//////////////////////////////////////////////////////////////////////

class TVFSMountPak : public TVFSMount {
 public:
  TVFSMountPak(const char * root);
  ~TVFSMountPak(void);

  virtual TVFSStream * fsFileOpen(const char * FileName, TVFSFileMode Mode);
  virtual void fsListFiles(const char * Path, TStringList * list);

  virtual bool fsFileExists(const char * FileName);
 private:
  TVFSStreamWin32 * FRoot;
  TPakHeader header;
  TPakEntry * directory;
  int numFiles;
  int getDirectoryIndex(const char * FileName);
};

//////////////////////////////////////////////////////////////////////

class TVFSStreamPak : public TVFSStream {
 private:
  TVFSStream * mount;
  int FOffset;
  PPakEntry info;
 public:
  TVFSStreamPak(PPakEntry entry, TVFSStream * pak);
  ~TVFSStreamPak(void);

  virtual int Read(void * buffer, int count);
  virtual int Write(const void * buffer, int count);
  virtual int Seek(int offset, TVFSOrigin origin);
  virtual bool setSize(int newSize);
};

//////////////////////////////////////////////////////////////////////

#endif

//////////////////////////////////////////////////////////////////////
