//////////////////////////////////////////////////////////////////////

#include <windows.h>

#include "system.h"
#include "jfilesystem.h"
#include "vfsWin32.h"

//////////////////////////////////////////////////////////////////////

bool FileExists(const char * FileName) {
  char buffer[2048];
  j_stringFill(buffer, 0, 2048);
  j_stringCopy(FileName, buffer, 2047);

  vfsTranslatePath(buffer, '\\');
  return GetFileAttributes(buffer) != 0xFFFFFFFF;
}

//////////////////////////////////////////////////////////////////////
// TVFSMountWin32 ////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

TVFSMountWin32::TVFSMountWin32(const char * root) {
  int length = j_stringLength(root);
  FRoot = new char[length+2];
  j_stringCopy(root, FRoot, length+1);
  FRoot[length] = '\\';
  FRoot[length+1] = 0;

  FValid = true;
}

//////////////////////////////////////////////////////////////////////

TVFSMountWin32::~TVFSMountWin32(void) {
  delete [] FRoot;
}

//////////////////////////////////////////////////////////////////////

bool TVFSMountWin32::fsFileExists(const char * FileName) {
  char buffer[2048];

  // Fixme: skankadelic
  j_stringFill(buffer, 0, 2048);
  j_stringCombine(FRoot, FileName, buffer, 2047);
  int length = j_stringLength(buffer);
  buffer[length] = 0;
  vfsTranslatePath(buffer, '\\');

  return GetFileAttributes(buffer) != 0xFFFFFFFF;
}

//////////////////////////////////////////////////////////////////////

TVFSStream * TVFSMountWin32::fsFileOpen(const char * FileName, TVFSFileMode Mode) {
  char buffer[2048];
  j_stringFill(buffer, 0, 2048);
  j_stringCombine(FRoot, FileName, buffer, 2047);
  vfsTranslatePath(buffer, '\\');
  return new TVFSStreamWin32(buffer, Mode);
}

//////////////////////////////////////////////////////////////////////

void TVFSMountWin32::fsListFiles(const char * Path, TStringList * list) {
  char buffer[2048];
  j_stringFill(buffer, 0, 2048);
  j_stringCombine(FRoot, Path, buffer, 2044);
  int length = j_stringLength(buffer);
  buffer[length] = '*';
  buffer[length+1] = '.';
  buffer[length+2] = '*';
  buffer[length+3] = 0;

  int res;
  WIN32_FIND_DATA FindData;
  HANDLE FindHandle;
  int ExcludeAttr = (~(faReadOnly | faHidden | faArchive)) & (faHidden | faSysFile | faVolumeID | faDirectory);

  // Correct the path delimiters
  vfsTranslatePath(buffer, '\\');

  // Find the first file in the directory
  FindHandle = FindFirstFile(buffer, &FindData);
  if (FindHandle == INVALID_HANDLE_VALUE) res = 1;

  res = 0;
  while (!res) {
    // Pair out files with bad attributes
    while (FindData.dwFileAttributes & ExcludeAttr)
      if (!FindNextFile(FindHandle, &FindData)) {
        res = 1;
        break;
      }
    if (res) break;

    // Add it to the list, if unique
    if (list->IndexOf(FindData.cFileName) < 0) list->Add(FindData.cFileName);

    // Look for the next file
    if (!FindNextFile(FindHandle, &FindData)) break;
  }

  // Release the windows find handle
  if (FindHandle != INVALID_HANDLE_VALUE) {
    FindClose(FindHandle);
    FindHandle = INVALID_HANDLE_VALUE;
  }
}

//////////////////////////////////////////////////////////////////////
// TVFSStreamWin32 ///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

int TVFSStreamWin32::Read(void * buffer, int count) {
  DWORD Result;
  ReadFile(FHandle, buffer, count, &Result, NULL);
  return Result;
}

//////////////////////////////////////////////////////////////////////

int TVFSStreamWin32::Write(const void * buffer, int count) {
  DWORD Result;
  WriteFile(FHandle, buffer, count, &Result, NULL);
  return Result;
}

//////////////////////////////////////////////////////////////////////

int TVFSStreamWin32::Seek(int offset, TVFSOrigin origin) {
  int winOrigin;

  switch (origin) {
    case vfsFromCurrent:
      winOrigin = FILE_CURRENT;
      break;
    case vfsFromEnd:
      winOrigin = FILE_END;
      break;
    default:
      winOrigin = FILE_BEGIN;
  }

  return SetFilePointer(FHandle, offset, NULL, winOrigin);
}

//////////////////////////////////////////////////////////////////////

bool TVFSStreamWin32::setSize(int newSize) {
  Seek(newSize, vfsFromBeginning);
  return SetEndOfFile(FHandle) != 0;
}

//////////////////////////////////////////////////////////////////////

const uint32 AccessMode[4] = {GENERIC_READ, GENERIC_WRITE, GENERIC_READ | GENERIC_WRITE, 0};
const uint32 ShareMode[8] = {0, 0, FILE_SHARE_READ, FILE_SHARE_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, 0, 0, 0};

//////////////////////////////////////////////////////////////////////

TVFSStreamWin32::TVFSStreamWin32(const char * FileName, int Mode) {
  if (Mode == vfsCreate)
    FHandle = CreateFile(FileName, GENERIC_READ | GENERIC_WRITE, 0,
                         NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
  else
    FHandle = CreateFile(FileName, AccessMode[Mode & 3], ShareMode[(Mode >> 4) & 0x7],
                         NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

  FValid = FHandle != INVALID_HANDLE_VALUE;
}

//////////////////////////////////////////////////////////////////////

TVFSStreamWin32::~TVFSStreamWin32(void) {
  if (FHandle >= 0) CloseHandle(FHandle);
}

//////////////////////////////////////////////////////////////////////
