//////////////////////////////////////////////////////////////////////
// vfsZipFile.cpp -- Implementation for the .ZIP archive VFS plugin //
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Includes //////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

#include <zlib.h>
#include "unzip.h"
#include "vfsZipFile.h"

//////////////////////////////////////////////////////////////////////
// TVFSMountZip //////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

TVFSMountZip::TVFSMountZip(const char * root) {
  FZipFile = unzOpen(root);
  FValid = FZipFile != NULL;
}

//////////////////////////////////////////////////////////////////////

TVFSMountZip::~TVFSMountZip(void) {
  if (FZipFile != NULL) unzClose(FZipFile);
}

//////////////////////////////////////////////////////////////////////

bool TVFSMountZip::fsFileExists(const char * FileName) {
  char buffer[2048];

  // Fixme: skankadelic
  j_stringFill(buffer, 0, 2048);
  j_stringCopy(FileName, buffer, 2047);
  int length = j_stringLength(buffer);
  buffer[length] = 0;
  vfsTranslatePath(buffer, '/');

  return unzLocateFile(FZipFile, buffer, 2) == UNZ_OK;
}

//////////////////////////////////////////////////////////////////////

TVFSStream * TVFSMountZip::fsFileOpen(const char * FileName, TVFSFileMode Mode) {
  bool valid;
  char buffer[2048];

  j_stringFill(buffer, 0, 2048);
  j_stringCopy(FileName, buffer, 2047);
  vfsTranslatePath(buffer, '/');


  // The zip file system is read-only
  switch (Mode) {
    case vfsOpenWrite:
    case vfsOpenReadWrite:
    case vfsCreate:
      return NULL;
  }

  // Try to find and open the file
  valid = unzLocateFile(FZipFile, buffer, 2) == UNZ_OK;
  valid = valid && (unzOpenCurrentFile(FZipFile) == UNZ_OK);
  if (!valid) return NULL;

  // Read in the file
  TVFSStream * Result = new TVFSStreamZip(FZipFile);

  // Clean up and return
  unzCloseCurrentFile(FZipFile);
  return Result;
}

//////////////////////////////////////////////////////////////////////

// fixme: cracktastic, god I hate string handling in C, its just, just
// soooooo dirty...
// This is a very good example of how you can write very cryptic C code
// without even trying, the for(a;b;c) construct is just *asking* to be
// (ab)used
void TVFSMountZip::fsListFiles(const char * Path, TStringList * list) {
  int res;
  int length;
  char buffer[2048];
  char buffer2[256];

  // Correct the path delimiters
  j_stringFill(buffer, 0, 2048);
  j_stringCopy(Path, buffer, 2047);
  vfsTranslatePath(buffer, '/');

  length = j_stringLength(buffer);

  res = unzGoToFirstFile(FZipFile);
  while (res == UNZ_OK) {
    // Get the file name of the current file into buffer
    buffer2[0] = 0;
    unzGetCurrentFileInfo(FZipFile, NULL, buffer2, 256, NULL, 0, NULL, 0);

    // To be a match, buffer must match path for length(path) chars,
    // and the remaining characters (there must be 1+) shouldn't
    // contain a directory (i.e. with no / characters)
    if (j_stringLength(buffer2) > length) {
      if ((length == 0) || (j_stringCompare(buffer, buffer2, false, length) == 0)) {
        char * p;
        for (p = &(buffer2[length]); (*p) && (*p != '/'); p++) ;
        if (!*p) if (list->IndexOf(buffer2) < 0) list->Add(buffer2);
      }
    }

    // Continue to the next file in the archive
    res = unzGoToNextFile(FZipFile);
  }
}

//////////////////////////////////////////////////////////////////////
// TVFSStreamZip /////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

int TVFSStreamZip::Read(void * buffer, int count) {
  j_copyMemory(buffer, ((char *)FBuffer + FPointer), count);
  FPointer += count;
  return count;
}

//////////////////////////////////////////////////////////////////////

int TVFSStreamZip::Write(const void * buffer, int count) {
  return 0;
}

//////////////////////////////////////////////////////////////////////

int TVFSStreamZip::Seek(int offset, TVFSOrigin origin) {
  switch (origin) {
    case vfsFromCurrent:
      FPointer += offset;
      break;
    case vfsFromEnd:
      FPointer = FLength + offset;
      break;
    default:
      FPointer = offset;
  }

  return FPointer;
}

//////////////////////////////////////////////////////////////////////

bool TVFSStreamZip::setSize(int newSize) {
  return false;
}

//////////////////////////////////////////////////////////////////////

TVFSStreamZip::TVFSStreamZip(void * zip) {
  unz_file_info info;

  FPointer = 0;
  FValid = unzGetCurrentFileInfo(zip, &info, NULL, 0, NULL, 0, NULL, 0) == UNZ_OK;
  FLength = info.uncompressed_size;

  if (FValid) {
    FBuffer = new byte[FLength];
    int bytesRead = unzReadCurrentFile(zip, FBuffer, FLength);
    if (FLength != bytesRead) {
      // There was a read error
      FValid = false;
      delete [] FBuffer;
      return;
    }
  }
}

//////////////////////////////////////////////////////////////////////

TVFSStreamZip::~TVFSStreamZip(void) {
  if (FValid) {
    delete [] FBuffer;
  }
}

//////////////////////////////////////////////////////////////////////