//////////////////////////////////////////////////////////////////////

#include "system.h"
#include "viewpoint.h"
#include "config.h"
#include "jmath.h"

//////////////////////////////////////////////////////////////////////
// renderMode2D - prepares OpenGL to render 2D stuff /////////////////
//////////////////////////////////////////////////////////////////////

void TViewpoint::renderMode2D(void) {
  // Set up the viewport
  glViewport(myLeft, myTop, myWidth, myHeight);

  // Set up an orthographic projection matrix
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, screenWidth, 0, screenHeight,  -1, 1);

  glScalef(1, -1, 1);
  // mover the origin from the bottom left corner
  // to the upper left corner
  glTranslatef(0, -1.0f*screenHeight, 0);
  
  //  glScalef(1.0f, -1.0f, 1.0f);
  //glTranslatef(0.0f, screenHeight, 0.0f);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  // Tune the state
  glDisable(GL_DEPTH_TEST);
}

//////////////////////////////////////////////////////////////////////
// renderMode3D - prepares OpenGL to render 3D stuff /////////////////
//////////////////////////////////////////////////////////////////////

void TViewpoint::renderMode3D(float nearDistance, float farDistance) {
  // Set up the viewport
  glViewport(myLeft, myTop, myWidth, myHeight);

  // Set up a perspective projection matrix
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  if (myHeight == 0) myHeight = 1;
  gluPerspective(config::verticalFOV, myWidth/(float)myHeight, nearDistance, farDistance);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  // Tune the state
  glEnable(GL_DEPTH_TEST);
}

//////////////////////////////////////////////////////////////////////

void TViewpoint::setPosition(int left, int top) {
  myLeft = left;
  myTop = top;
}

//////////////////////////////////////////////////////////////////////

void TViewpoint::setDimensions(int width, int height) {
  myWidth = width;
  myHeight = height;
}

//////////////////////////////////////////////////////////////////////

void TViewpoint::lookat(void) {
  gluLookAt(pos[0], pos[1], pos[2],
            pos[0]+j_cos(theta*0.03491f)*j_cos(phi*0.03491f),
            pos[1]+j_sin(theta*0.03491f)*j_cos(phi*0.03491f),
            pos[2]+j_sin(phi*0.03491f),
            0.0f, 0.0f, 1.0f);
}

//////////////////////////////////////////////////////////////////////

void TViewpoint::lookOnly(void) {
  gluLookAt(0.0f, 0.0f, 0.0f,
            j_cos(theta*0.03491f)*j_cos(phi*0.03491f),
            j_sin(theta*0.03491f)*j_cos(phi*0.03491f),
            j_sin(phi*0.03491f),
            0.0f, 0.0f, 1.0f);
}

//////////////////////////////////////////////////////////////////////
// The camera follows an object from behind at a distance of
// focalLength with an elevation of phi in radians, where phi is
//   pi/2*(cameraElevation - cameraFlyback*speedRatio), and
// speedRatio is ratio of speed to maximum speed of the ship
// being followed (0.0 to 1.0)

// Makes the camera active
void TViewpoint::lookAtObject(void) {
    // Calculate the angle of elevation (1.99 to avoid being
    // coincident with the up vector (thus flipping the scene)
    float speedRatio = 0.0f; // fixme, only applicable to ships, etc...: FTarget->velocity.length() / FTarget->maxSpeed;
    phi = (FElevation - FFlyback*speedRatio)*180.0f/1.99f;


    // Fixme: once again, code that has no application to the current shit!
    //if (FRotate) theta = FTarget->heading; else theta = 0.0f;
    theta = 0.0f;

    // Set up the modelview matrix
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

  gluLookAt(pos[0]-FFocalLength*j_cos(theta*0.03491f)*j_cos(phi*0.03491f), 
            pos[1]+FFocalLength*j_sin(phi*0.03491f), 
            pos[2]-FFocalLength*j_sin(theta*0.03491f)*j_cos(phi*0.03491f),
            pos[0],
            pos[1],
            pos[2],
            0.0f, 1.0f, 0.0f);
}
