//////////////////////////////////////////////////////////////////////
unit drawActions; ////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
interface ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

uses
  mazeTypes;

//////////////////////////////////////////////////////////////////////

type
  TDrawAction = class(TUndoableAction)
  protected
    myX, myY, myLayer, mySize: integer;
    myColor: integer;
    backup: array of integer;
  public
    constructor Create(x, y, layer: integer; penSize: integer; color: integer);
    destructor Destroy; override;
    procedure Undo(maze: TMaze); override;
    procedure Redo(maze: TMaze); override;
    class function Name: string; override;
  end;

//////////////////////////////////////////////////////////////////////
implementation //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// TDrawAction ///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

constructor TDrawAction.Create(x, y, layer, penSize, color: integer);
begin
  myX := x;
  myY := y;
  myLayer := layer;
  mySize := penSize;
  myColor := color;
  SetLength(backup, mySize*mySize);
end;

//////////////////////////////////////////////////////////////////////

destructor TDrawAction.Destroy;
begin
  inherited;
end;

//////////////////////////////////////////////////////////////////////

class function TDrawAction.Name: string;
begin
  Result := 'Drawing';
end;

//////////////////////////////////////////////////////////////////////

procedure TDrawAction.Redo(maze: TMaze);
var
  x, y: integer;
begin
  // Make a backup and then paint with our new color
  for x := 0 to mySize - 1 do
    for y := 0 to mySize - 1 do begin
      backup[x+y*mySize] := maze.ReadCell(x+myX, y+myY, myLayer);
      maze.WriteCell(x+myX, y+myY, myLayer, myColor);
    end;
end;

//////////////////////////////////////////////////////////////////////

procedure TDrawAction.Undo(maze: TMaze);
var
  x, y: integer;
begin
  // Restore what we painted over, using the backup copy
  for x := 0 to mySize - 1 do
    for y := 0 to mySize - 1 do begin
      maze.WriteCell(x+myX, y+myY, myLayer, backup[x+y*mySize]);
    end;
end;

//////////////////////////////////////////////////////////////////////

end.

//////////////////////////////////////////////////////////////////////
