//////////////////////////////////////////////////////////////////////
unit edform1; ////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
interface ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Menus, ImgList, ComCtrls, ToolWin,
  Math, Contnrs, ActnList, objUtils, Comobj, soundEditor,
  mazeTypes, itemActions, zoneActions, drawActions, resourceActions;

//////////////////////////////////////////////////////////////////////

type
  TTool = (toolNone, toolDrawWall, toolDrawFloor, toolDrawZone, toolPlaceStart, toolPlaceExit, toolPlaceItem);

  TMapEditorForm = class(TForm)
    MainMenu: TMainMenu;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    scrollbox: TScrollBox;
    paintbox: TPaintBox;
    mFile: TMenuItem;
    status: TStatusBar;
    images: TImageList;
    mFileNewMap: TMenuItem;
    mFileSave: TMenuItem;
    mFileSaveAs: TMenuItem;
    mFileExit: TMenuItem;
    N1: TMenuItem;
    mEdit: TMenuItem;
    mEditUndo: TMenuItem;
    mEditRedo: TMenuItem;
    Open1: TMenuItem;
    N2: TMenuItem;
    actions: TActionList;
    aToolDrawWall: TAction;
    aToolClearFloor: TAction;
    aToolAddSound: TAction;
    aToolPlaceStart: TAction;
    aToolPlaceExit: TAction;
    pages: TPageControl;
    zoneTab: TTabSheet;
    mazeTab: TTabSheet;
    entityTab: TTabSheet;
    zoneList: TListBox;
    zoneBottomPanel: TPanel;
    zoneDelete: TButton;
    zoneAdd: TButton;
    zoneRename: TButton;
    toolbar: TToolBar;
    wallDrawWall: TToolButton;
    wallDrawFloor: TToolButton;
    wallPlaceStart: TToolButton;
    wallPlaceExit: TToolButton;
    colorDialog: TColorDialog;
    zoneColor: TButton;
    Zoom1: TMenuItem;
    N16x1: TMenuItem;
    N32x1: TMenuItem;
    N64x1: TMenuItem;
    Label1: TLabel;
    description: TMemo;
    N12x1: TMenuItem;
    N24x1: TMenuItem;
    N48x1: TMenuItem;
    Panel1: TPanel;
    entityDelete: TButton;
    entityAdd: TButton;
    entityRename: TButton;
    entityEdit: TButton;
    entityList: TListBox;
    entityPickColor: TButton;
    procedure FormCreate(Sender: TObject);
    procedure CreateNewMap(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure paintboxMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure OpenMap(Sender: TObject);
    procedure paintboxPaint(Sender: TObject);
    procedure SaveMap(Sender: TObject);
    procedure SaveMapAs(Sender: TObject);
    procedure ExitProgram(Sender: TObject);
    procedure paintboxMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure paintboxMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure mEditUndoClick(Sender: TObject);
    procedure mEditRedoClick(Sender: TObject);
    procedure pickTool(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);

    procedure zoneChanged(Sender: TObject);
    procedure zoneAddClick(Sender: TObject);
    procedure zoneRenameClick(Sender: TObject);
    procedure zoneDeleteClick(Sender: TObject);
    procedure zoneListDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure zoneEditClick(Sender: TObject);
    procedure changeZoomClick(Sender: TObject);
    procedure pagesChange(Sender: TObject);
    procedure entityChanged(Sender: TObject);
    procedure entityListDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure entityAddClick(Sender: TObject);
    procedure entityDeleteClick(Sender: TObject);
    procedure entityRenameClick(Sender: TObject);
    procedure entityEditClick(Sender: TObject);
    procedure paintboxDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure paintboxDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure entityPickColorClick(Sender: TObject);
    procedure mazeTabShow(Sender: TObject);
    procedure descriptionChange(Sender: TObject);
  private
    { Private declarations }
    currentTool: TTool;
    mouseIsDown: boolean;
    currentFilename: string;
    maze: TMaze;
    undoIndex: integer; // points to the last action done
    sequenceID: integer;

    entitySelection: integer;

    undoStack: TObjectList;

    procedure setScaler(scale: integer);
    procedure MapLoaded;
    function VerifyLosingChanges(const reason: string): boolean;

    // Takes care of killing undone things on the stack,
    // since it's a stack, not a tree!
    procedure PostAction(action: TUndoableAction);
    procedure EntityToolMovement(ix, iy: integer);
    procedure StatusMessage(msg: string);

    procedure RenameResource(list: TListBox);
    procedure ColorResource(list: TListBox);
  public
    { Public declarations }
  end;

//////////////////////////////////////////////////////////////////////

var
  MapEditorForm: TMapEditorForm;

//////////////////////////////////////////////////////////////////////
implementation ///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

uses
  NewMapForm, soundEditorForm, StartWizard;

//////////////////////////////////////////////////////////////////////

{$R *.DFM}

//////////////////////////////////////////////////////////////////////
// TForm1 ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.FormCreate(Sender: TObject);
begin
  currentTool := toolNone;
  sequenceID := 0;
  mouseIsDown := false;

  maze := TMaze.Create(16, 16, 1.0);
  undoStack := TObjectList.Create;
  currentFilename := '';

  scrollbox.DoubleBuffered := true;

  MapLoaded();
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.CreateNewMap(Sender: TObject);
begin
  if VerifyLosingChanges('creating a new map') then begin
    if NewMapDialog.ShowModal = mrOk then begin
      // Create the new map
      maze.Free;
      maze := TMaze.Create(NewMapDialog.outWidth, NewMapDialog.outHeight, NewMapDialog.outCellSize);
      MapLoaded();
    end;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.FormDestroy(Sender: TObject);
begin
  maze.Free;
  undoStack.Free;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.MapLoaded;
begin
  // Flush the undo stack
  undoIndex := -1;
  undoStack.Clear;
  mEditUndo.Enabled := false;
  mEditRedo.Enabled := false;

  // Reconfigure the paint box
  changeZoomClick(N16x1);

  // Refresh the zone list
  maze.zones.UpdateDisplay(zoneList);
  maze.entities.UpdateDisplay(entityList);

  description.Text := maze.description;
  
  // Select a tool
  pages.ActivePageIndex := 0;
  pagesChange(pages);

  // Reset the selection
  entitySelection := INVALID_UID;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.OpenMap(Sender: TObject);
begin
  if VerifyLosingChanges('opening a different map') then begin
    OpenDialog.FileName := currentFilename;
    if OpenDialog.Execute then begin
      currentFilename := OpenDialog.FileName;
      if maze.LoadFromFile(OpenDialog.FileName) then begin
        MapLoaded();
      end else
        ShowMessage('Could not load the file "' + OpenDialog.FileName + '".  It may not exist, or it may be for an older version of aMaze.');
    end;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.paintboxPaint(Sender: TObject);
begin
  // Render the maze
  maze.Render(paintbox.Canvas);
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.SaveMap(Sender: TObject);
begin
  if currentFilename = '' then
    SaveMapAs(Sender)
  else
    maze.SaveToFile(currentFilename);
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.SaveMapAs(Sender: TObject);
begin
  SaveDialog.FileName := currentFilename;
  if SaveDialog.Execute then begin
    currentFilename := SaveDialog.FileName;
    SaveMap(Sender);
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.ExitProgram(Sender: TObject);
begin
  Close;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.paintboxMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  mouseIsDown := true;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.paintboxMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  paintboxMouseMove(Sender, Shift, X, Y);
  mouseIsDown := false;
  Inc(sequenceID);
  entitySelection := INVALID_UID;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.paintboxMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
  layer, color: integer;
begin
  // fixme: mmh, totally bad.  this code is a case study in stupid
  if not mouseIsDown then Exit;

  color := -1;
  layer := 0;

  case currentTool of
    toolDrawFloor: color := MATERIAL_FLOOR;
    toolDrawWall: color := MATERIAL_WALL;
    toolPlaceStart: color := MATERIAL_ENTRANCE;
    toolPlaceExit: color := MATERIAL_EXIT;
    toolDrawZone: begin
      // Zones
      if zoneList.ItemIndex >= 0 then
        color := (zoneList.Items.Objects[zoneList.ItemIndex] as TZone).uid;
      layer := 1;
    end;
    toolPlaceItem: begin
      EntityToolMovement(x, y);
      Exit;
    end;
  end;

  if (color >= 0) then begin
    x := x div maze.scale;
    y := y div maze.scale;
    if color <> maze.ReadCell(x, y, layer) then
      PostAction(TDrawAction.Create(x, y, layer, 1, color));
  end else begin
    // it's one of the other unsupported tools or a broken color
    StatusMessage('No tool selected!');
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.mEditUndoClick(Sender: TObject);
var
  i, id: integer;
  action: TUndoableAction;
begin
  // Undo everything with the same sequence index as the top of the stack
  i := undoIndex;
  if (i < 0) or (i >= undoStack.Count) then Exit;
  id := TUndoableAction(undoStack.Items[i]).sequenceID;
  while (i >= 0) and
        (id = TUndoableAction(undoStack.Items[i]).sequenceID) do begin
    TUndoableAction(undoStack.Items[i]).Undo(maze);
    Dec(i);
  end;
  undoIndex := i;
  paintbox.Invalidate;

  // Update the undo / redo options
  mEditRedo.Enabled := true;

  // Point to the last action done (if any)
  if undoIndex < 0 then begin
    mEditUndo.Enabled := false;
    mEditUndo.Caption := '&Undo';
  end else begin
    action := TUndoableAction(undoStack.Items[undoIndex]);
    mEditUndo.Caption := '&Undo ' + action.Name;
  end;

  // Point to the most recently undone action
  action := TUndoableAction(undoStack.Items[undoIndex+1]);
  mEditRedo.Caption := '&Redo ' + action.Name;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.mEditRedoClick(Sender: TObject);
var
  id, i: integer;
  action: TUndoableAction;
begin
  // Activate all the actions in the stack with the same sequence ID
  i := undoIndex + 1;
  if (i < 0) or (i >= undoStack.Count) then Exit;
  id := TUndoableAction(undoStack.Items[i]).sequenceID;
  while (i < undoStack.Count) and
        (id = TUndoableAction(undoStack.Items[i]).sequenceID) do begin
    TUndoableAction(undoStack.Items[i]).Redo(maze);
    Inc(i);
  end;
  undoIndex := i - 1;
  paintbox.Invalidate;

  // Update the undo / redo options
  mEditUndo.Enabled := true;

  // Point to the most recently undone action (if any)
  if undoIndex = undoStack.Count - 1 then begin
    mEditRedo.Enabled := false;
    mEditRedo.Caption := '&Redo';
  end else begin
    action := TUndoableAction(undoStack.Items[undoIndex+1]);
    mEditRedo.Caption := '&Redo ' + action.Name;
  end;

  // Point to the last action done
  action := TUndoableAction(undoStack.Items[undoIndex]);
  mEditUndo.Caption := '&Undo ' + action.Name;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.PostAction(action: TUndoableAction);
begin
  // Delete everything above the current position in the stack
  while (undoStack.Count > undoIndex+1) do
    undoStack.Delete(undoStack.Count - 1);
  action.sequenceID := sequenceID;

  // Post and perform the new action
  undoStack.Add(action);
  mEditRedoClick(self);

  // Update the undo / redo menu items
  mEditUndo.Caption := '&Undo ' + action.Name;

  mEditRedo.Enabled := false;
  mEditRedo.Caption := '&Redo';

  // Refresh the display
  zoneList.Invalidate;
  entityList.Invalidate;
  paintbox.Invalidate;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.pickTool(Sender: TObject);
begin
  case (Sender as TComponent).Tag of
    1: begin
      currentTool := toolDrawWall;
      wallDrawWall.Down := true;
      StatusMessage('Use the mouse to draw walls');
    end;
    2: begin
      currentTool := toolDrawFloor;
      wallDrawFloor.Down := true;
      StatusMessage('Use the mouse to remove walls (draw floors)');
    end;
    4: begin
      currentTool := toolPlaceStart;
      wallPlaceStart.Down := true;
      StatusMessage('Use the mouse to place starting points (entrances)');
    end;
    5: begin
      currentTool := toolPlaceExit;
      wallPlaceExit.Down := true;
      StatusMessage('Use the mouse to place exits');
    end;
  else
    currentTool := toolNone;
  end;
end;

//////////////////////////////////////////////////////////////////////

function TMapEditorForm.VerifyLosingChanges(const reason: string): boolean;
var
  res: word;
begin
  Result := true;
  if maze.modified then begin
    // ask about saving
    res := MessageDlg('Would you like to save your progress before ' + reason + '?', mtConfirmation, mbYesNoCancel, 0);
    if res = mrYes then
      SaveMap(nil)
    else if res = mrCancel then
      Result := false;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := VerifyLosingChanges('exiting the program');
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
//
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.EntityToolMovement(ix, iy: integer);
var
  entity, changes: TEntity;
  x, y: single;
begin
  x := maze.cellSize * (ix / maze.scale - 0.5);
  y := maze.cellSize * (iy / maze.scale - 0.5);

  if entitySelection = INVALID_UID then begin
    // Try to select an entity
    entity := maze.entities.FindInColumn(x, y, maze.cellSize*0.8);
    if entity <> nil then entitySelection := entity.uid;
  end else begin
    // Find the entity and move it
    entity := maze.entities.Find(entitySelection) as TEntity;

    if entity <> nil then begin
      changes := TMazeSound.Create; // fixme: hard-coded!
      changes.Assign(entity);
      changes.x := x;
      changes.y := y;
      PostAction(EntityEditAction.Create(entity, changes, TMazeSound.Create)); // fixme: hardcoded
    end;

    paintbox.Invalidate;

    // Moving around
    //  PostAction(TSoundAction.Create());
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.FormShow(Sender: TObject);
begin
  if StartWizardForm.ShowModal = mrNo then
    CreateNewMap(Sender)
  else
    OpenMap(Sender);
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.zoneChanged(Sender: TObject);
var
  selected: boolean;
begin
  selected := zoneList.ItemIndex >= 0;
  zoneColor.Enabled := selected;
  zoneRename.Enabled := selected;
  zoneDelete.Enabled := selected;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.zoneListDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  zone: TZone;
  list: TListBox;
  color: TColor;
  right: integer;
  flags: integer;
  st: string;
begin
  list := Control as TListBox;
  if (Index > -1) and (Index < list.Items.Count) then begin
    zone := list.Items.Objects[Index] as TZone;

    // Draw a colored rectangle
    color := list.Canvas.Brush.Color;
    list.Canvas.FillRect(Rect);
    list.Canvas.Brush.Color := zone.color;
    right := Rect.Left+(Rect.Bottom-Rect.Top)-2;
    list.Canvas.FillRect(Classes.Rect(Rect.Left+1, Rect.Top+1, right, Rect.Bottom-1));
    list.Canvas.Brush.Color := color;

    // Draw the zone number
    list.Canvas.Brush.Style := bsClear;
    flags := DrawTextBiDiModeFlags(DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
    rect.Left := rect.Left + 3;
    st := IntToStr(index);
    DrawText(list.Canvas.Handle, PChar(st), Length(st), rect, flags);

    // Draw the zone name
    rect.Left := right + 2;
    DrawText(list.Canvas.Handle, PChar(list.Items[Index]), Length(list.Items[Index]), rect, flags);

    list.Canvas.Brush.Style := bsSolid;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.zoneAddClick(Sender: TObject);
begin
  Inc(sequenceID);
  PostAction(ZoneCreationAction.Create(maze, zoneList));
  zoneChanged(Sender);
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.zoneRenameClick(Sender: TObject);
begin
  RenameResource(zoneList);
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.zoneDeleteClick(Sender: TObject);
begin
  if zoneList.ItemIndex >= 0 then begin
    Inc(sequenceID);
    PostAction(ResourceDeletionAction.Create(zoneList.Items.Objects[zoneList.ItemIndex] as TMapResource, zoneList));
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.zoneEditClick(Sender: TObject);
begin
  ColorResource(zoneList);
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.changeZoomClick(Sender: TObject);
var
  item: TMenuItem;
begin
  if Sender is TMenuItem then begin
    item := Sender as TMenuItem;
    setScaler(item.Tag);
    item.Checked := true;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.setScaler(scale: integer);
begin
  maze.scale := Max(scale, 4);
  paintbox.Width := maze.width * maze.scale;
  paintbox.Height := maze.height * maze.scale;
  paintbox.Invalidate;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.StatusMessage(msg: string);
begin
  status.Panels[2].Text := msg;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.pagesChange(Sender: TObject);
begin
  case pages.ActivePageIndex of
    0: begin
      wallDrawWall.Click;
    end;
    1: begin
      currentTool := toolDrawZone;
    end;
    2, 3: begin
      currentTool := toolPlaceItem;
    end;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.entityChanged(Sender: TObject);
var
  selected: boolean;
begin
  selected := entityList.ItemIndex >= 0;
  entityEdit.Enabled := selected;
  entityRename.Enabled := selected;
  entityDelete.Enabled := selected;
  entityPickColor.Enabled := selected;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.entityListDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  resource: TMapResource;
  list: TListBox;
  color: TColor;
  right: integer;
  flags: integer;
  st: string;
begin
  list := Control as TListBox;
  if (Index > -1) and (Index < list.Items.Count) then begin
    resource := list.Items.Objects[Index] as TMapResource;

    // Draw a colored rectangle
    color := list.Canvas.Brush.Color;
    list.Canvas.FillRect(Rect);
    list.Canvas.Brush.Color := resource.color;
    right := Rect.Left+(Rect.Bottom-Rect.Top)-2;
    list.Canvas.FillRect(Classes.Rect(Rect.Left+1, Rect.Top+1, right, Rect.Bottom-1));
    list.Canvas.Brush.Color := color;

    // Draw the zone number
    list.Canvas.Brush.Style := bsClear;
    flags := DrawTextBiDiModeFlags(DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
    rect.Left := rect.Left + 3;
    st := 'S';
    DrawText(list.Canvas.Handle, PChar(st), Length(st), rect, flags);

    // Draw the zone name
    rect.Left := right + 2;
    DrawText(list.Canvas.Handle, PChar(list.Items[Index]), Length(list.Items[Index]), rect, flags);

    list.Canvas.Brush.Style := bsSolid;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.entityAddClick(Sender: TObject);
begin
  Inc(sequenceID);
  PostAction(EntityCreationAction.Create(maze, entityList));
  entityChanged(Sender);
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.entityDeleteClick(Sender: TObject);
begin
  if entityList.ItemIndex >= 0 then begin
    Inc(sequenceID);
    PostAction(ResourceDeletionAction.Create(entityList.Items.Objects[entityList.ItemIndex] as TMapResource, entityList));
    entityChanged(Sender);
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.entityRenameClick(Sender: TObject);
begin
  RenameResource(entityList);
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.entityEditClick(Sender: TObject);
var
  entity: TMazeSound; // fixme: hard-coded
begin
  if entityList.ItemIndex >= 0 then begin
    entity := entityList.Items.Objects[entityList.ItemIndex] as TMazeSound;

    SoundEditForm.editor.SetSound(entity);
    if SoundEditForm.ShowModal = mrOk then begin
      // Send it off to the action
      Inc(sequenceID);
      PostAction(EntityEditAction.Create(entity, SoundEditForm.editor.GetSound(), TMazeSound.Create)); // fixme: hardcoded
    end;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.RenameResource(list: TListBox);
var
  resource: TMapResource;
  name: string;
begin
  if list.ItemIndex >= 0 then begin
    resource := list.Items.Objects[list.ItemIndex] as TMapResource;
    name := InputBox('aMaze Editor', 'What would you like to rename "' + resource.name + '" to?', resource.name);
    if resource.name <> name then begin
      Inc(sequenceID);
      PostAction(ResourceRenameAction.Create(resource, name, list));
    end;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.ColorResource(list: TListBox);
var
  resource: TMapResource;
  color: TColor;
begin
  if list.ItemIndex >= 0 then begin
    resource := list.Items.Objects[list.ItemIndex] as TMapResource;
    colorDialog.Color := resource.color;
    if colorDialog.Execute then begin
      color := ColorToRGB(colorDialog.color);
      if color <> resource.color then begin
        Inc(sequenceID);
        PostAction(ResourcePickColorAction.Create(resource, color));
      end;
    end;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.paintboxDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  list: TListBox;
begin
  if Source is TListBox then begin
    list := Source as TListBox;
    if (list.ItemIndex >= 0) then begin
      self.entitySelection := (list.Items.Objects[list.ItemIndex] as TMapResource).uid;
      EntityToolMovement(x, y);
    end;
    paintbox.Invalidate;
  end;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.paintboxDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := true;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.entityPickColorClick(Sender: TObject);
begin
  ColorResource(entityList);
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.mazeTabShow(Sender: TObject);
begin
  description.Text := maze.description;
end;

//////////////////////////////////////////////////////////////////////

procedure TMapEditorForm.descriptionChange(Sender: TObject);
begin
  maze.description := description.Text;
end;

//////////////////////////////////////////////////////////////////////

initialization
  Randomize;
end.

//////////////////////////////////////////////////////////////////////

