//////////////////////////////////////////////////////////////////////
unit itemActions; ////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
interface ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

uses
  Math, StdCtrls, mazeTypes, resourceActions;

//////////////////////////////////////////////////////////////////////

type
  EntityCreationAction = class(ResourceDeletionAction)
  public
    constructor Create(maze: TMaze; list: TListBox);
    procedure Undo(maze: TMaze); override;
    procedure Redo(maze: TMaze); override;
    class function Name: string; override;
  end;

  EntityEditAction = class(TUndoableAction)
  protected
    myTarget, backup, changed: TEntity;
  public
    // The edit action will own the changes and dummy entities
    constructor Create(target: TEntity; changes, dummy: TEntity);
    destructor Destroy; override;
    procedure Undo(maze: TMaze); override;
    procedure Redo(maze: TMaze); override;
    class function Name: string; override;
  end;

//////////////////////////////////////////////////////////////////////
implementation ///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// EntityCreationAction //////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

constructor EntityCreationAction.Create(maze: TMaze; list: TListBox);
var
  entity: TEntity;
begin
  // Create a new zone
  entity := TMazeSound.Create; // fixme: only supporting sounds right now!
  entity.uid := getNewUID();
  entity.name := maze.entities.GetUniqueName('Sound');

  // Call the delete action constructor, but we're going to own it
  inherited Create(entity, list);
  owner := true;
end;

//////////////////////////////////////////////////////////////////////

class function EntityCreationAction.Name: string;
begin
  Result := 'Create Sound';
end;

//////////////////////////////////////////////////////////////////////

procedure EntityCreationAction.Redo(maze: TMaze);
begin
  // Redo the creation (i.e. undo the destruction)
  inherited Undo(maze);
end;

//////////////////////////////////////////////////////////////////////

procedure EntityCreationAction.Undo(maze: TMaze);
begin
  // Undo the creation, i.e. redo the destruction
  inherited Redo(maze);
end;

//////////////////////////////////////////////////////////////////////
// EntityEditAction //////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

constructor EntityEditAction.Create(target, changes, dummy: TEntity);
begin
  myTarget := target;
  backup := dummy;
  changed := changes;

  backup.uid := myTarget.uid;
  changed.uid := myTarget.uid;
end;

//////////////////////////////////////////////////////////////////////

destructor EntityEditAction.Destroy;
begin
  backup.Free;
  changed.Free;
  inherited;
end;

//////////////////////////////////////////////////////////////////////

class function EntityEditAction.Name: string;
begin
  Result := 'Edit';
end;

//////////////////////////////////////////////////////////////////////

procedure EntityEditAction.Redo(maze: TMaze);
begin
  backup.Assign(myTarget);
  myTarget.Assign(changed);
end;

//////////////////////////////////////////////////////////////////////

procedure EntityEditAction.Undo(maze: TMaze);
begin
  myTarget.Assign(backup);
end;

//////////////////////////////////////////////////////////////////////

end.

//////////////////////////////////////////////////////////////////////
