//////////////////////////////////////////////////////////////////////
unit zoneActions; ////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
interface ////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

uses
  Math, StdCtrls, mazeTypes, resourceActions;

//////////////////////////////////////////////////////////////////////

type
  ZoneCreationAction = class(ResourceDeletionAction)
  public
    constructor Create(maze: TMaze; list: TListBox);
    procedure Undo(maze: TMaze); override;
    procedure Redo(maze: TMaze); override;
    class function Name: string; override;
  end;

//////////////////////////////////////////////////////////////////////
implementation ///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// ZoneCreationAction ////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

constructor ZoneCreationAction.Create(maze: TMaze; list: TListBox);
var
  zone: TZone;
begin
  // Create a new zone
  zone := TZone.Create;
  zone.uid := getNewUID();
  zone.name := maze.zones.GetUniqueName('Zone');

  // Call the delete action constructor, but we're going to own it
  inherited Create(zone, list);
  owner := true;
end;

//////////////////////////////////////////////////////////////////////

class function ZoneCreationAction.Name: string;
begin
  Result := 'Create Zone';
end;

//////////////////////////////////////////////////////////////////////

procedure ZoneCreationAction.Redo(maze: TMaze);
begin
  // Redo the creation (i.e. undo the destruction)
  inherited Undo(maze);
end;

//////////////////////////////////////////////////////////////////////

procedure ZoneCreationAction.Undo(maze: TMaze);
begin
  // Undo the creation, i.e. redo the destruction
  inherited Redo(maze);
end;

//////////////////////////////////////////////////////////////////////

end.

//////////////////////////////////////////////////////////////////////
